@@ -25,6 +25,7 @@
 #include <sys/time.h>
 #include <time.h>
 #include <errno.h>
+#include <pthread.h>
 
 #ifdef HAVE_STDINT_H
 #include <stdint.h>
@@ -131,9 +132,13 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  m->add_plainly= 0;
  m->split_size= 0;
  strcpy(m->list_delimiter, "--");
- m->ino_behavior= 7;
+ m->ino_behavior= 1 | 2 | 4 | 32;      /* off:no_lsl_count */
  m->iso_level= 3;
+ m->iso_level_is_default= 1;
  m->do_joliet= 0;
+ m->do_hfsplus= 0;
+ m->do_fat= 0;
+ m->do_rockridge= 1;
  m->do_iso1999= 0;
  m->do_aaip= 0;
  m->do_md5= 0;
@@ -144,6 +149,7 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  m->scdbackup_tag_written[0]= 0;
  m->scdbackup_tag_listname[0]= 0;
  m->relax_compliance= 0;
+ m->allow_dir_id_ext_dflt= 1;
  m->rr_reloc_dir[0]= 0;
  m->rr_reloc_flags= 1;
  m->untranslated_name_len= 0;
@@ -167,6 +173,7 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  m->copyright_file[0]= 0;
  m->biblio_file[0]= 0;
  m->abstract_file[0]= 0;
+ strcpy(m->application_use, " ");
  m->session_logfile[0]= 0;
  m->session_lba= -1;
  m->session_blocks= 0;
@@ -175,6 +182,7 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  m->do_global_mode= 0;
  m->global_dir_mode= 0555;
  m->global_file_mode= 0444;
+ m->do_tao= 0;
  m->filters= NULL;
  m->filter_list_closed= 0;
  m->zlib_level_default= m->zlib_level= 6;
@@ -236,6 +244,7 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  m->boot_image_bin_path[0]= 0;
  m->boot_image_bin_form[0]= 0;
  m->boot_image_emul= 0;
+ m->boot_emul_default= 1;
  m->boot_image_load_size= 4 * 512;     /* hearsay out of libisofs/demo/iso.c */
  memset(m->boot_id_string, 0, sizeof(m->boot_id_string));
  memset(m->boot_selection_crit, 0, sizeof(m->boot_selection_crit));
@@ -253,11 +262,17 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  m->partition_offset= 0;
  m->partition_secs_per_head= 0;
  m->partition_heads_per_cyl= 0;
+ m->prep_partition[0]= 0;
+ m->efi_boot_partition[0]= 0;
  for(i= 0; i < Xorriso_max_appended_partitionS; i++) {
    m->appended_partitions[i]= NULL;
    m->appended_part_types[i]= 0;
  }
  m->ascii_disc_label[0]= 0;
+ m->grub2_sparc_core[0]= 0;
+ memset(m->hfsp_serial_number, 0, 8);
+ m->hfsp_block_size= 0;
+ m->apm_block_size= 0;
  m->vol_creation_time= 0;
  m->vol_modification_time= 0;
  m->vol_expiration_time= 0;
@@ -280,7 +295,9 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  m->do_restore_sort_lba= 0;
  m->do_strict_acl= 0;
  m->dialog= 0;
+ m->buffered_dialog= NULL;
  m->bsl_interpretation= 0;
+ m->sh_style_result= 0;
  m->search_mode= 0;
  m->structured_search= 1;
  m->do_iso_rr_pattern= 1;
@@ -290,8 +307,10 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  m->disk_exclusions= NULL;
  m->iso_rr_hidings= NULL;
  m->joliet_hidings= NULL;
+ m->hfsplus_hidings= NULL;
  m->disk_excl_mode= 1;
  m->use_stdin= 0;
+ m->tolerate_stdin_eof= 0;
  m->result_page_length= 0;
  m->result_page_width= 80;
  m->mark_text[0]= 0;
@@ -301,11 +320,26 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
    m->logfile_fp[i]= NULL;
  }
  m->pktlog_fp= NULL;
+ m->stderr_fp= NULL;
  for(i= 0; i < Xorriso_max_outlist_stacK; i++) {
    m->result_msglists[i]= NULL;
    m->info_msglists[i]= NULL;
    m->msglist_flags[i]= 0;
  }
+ m->lib_msg_queue_lock_ini= 0;
+ m->result_msglists_lock_ini= 0;
+ m->write_to_channel_lock_ini= 0;
+ m->msg_watcher_lock_ini= 0;
+ m->msg_watcher_state= 0;
+ m->msgw_result_handler= NULL;
+ m->msgw_result_handle= NULL;
+ m->msgw_info_handler= NULL;
+ m->msgw_info_handle= NULL;
+ m->msgw_stack_handle= -1;
+ m->msgw_msg_pending= 0;
+ m->msgw_fetch_lock_ini= 0;
+ m->msg_sieve= NULL;
+ m->msg_sieve_disabled= 0;
  m->msglist_stackfill= 0;
  m->status_history_max= Xorriso_status_history_maX;
  m->scsi_log= 0;
@@ -316,6 +350,7 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  Xorriso__text_to_sev(m->abort_on_text, &m->abort_on_severity, 0);
  m->abort_on_is_default= 1;
  m->problem_status= 0;
+ m->problem_status_lock_ini= 0;
  m->problem_status_text[0]= 0;
  m->errfile_log[0]= 0;
  m->errfile_mode= 0;
@@ -346,6 +381,7 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  m->insert_count= 0;
  m->insert_bytes= 0;
  m->error_count= 0;
+ m->launch_frontend_banned= 0;
  m->pacifier_style= 0;
  m->pacifier_interval= 1.0;
  m->pacifier_count= 0;
@@ -412,6 +448,9 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  if(ret<=0)
    goto failure;
  ret= Exclusions_new(&(m->joliet_hidings), 0);
+ if(ret<=0)
+   goto failure;
+ ret= Exclusions_new(&(m->hfsplus_hidings), 0);
  if(ret<=0)
    goto failure;
  Xorriso_relax_compliance(m, "default", 0);
@@ -419,6 +458,30 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  if(ret <= 0)
    goto failure;
  Xorriso_preparer_string(m, m->preparer_id, 1); /* avoids library calls */
+ ret= pthread_mutex_init(&(m->lib_msg_queue_lock), NULL);
+ if(ret != 0)
+   goto failure;
+ m->lib_msg_queue_lock_ini= 1;
+ ret= pthread_mutex_init(&(m->result_msglists_lock), NULL);
+ if(ret != 0)
+   goto failure;
+ m->result_msglists_lock_ini= 1;
+ ret= pthread_mutex_init(&(m->write_to_channel_lock), NULL);
+ if(ret != 0)
+   goto failure;
+ m->result_msglists_lock_ini= 1;
+ ret= pthread_mutex_init(&(m->problem_status_lock), NULL);
+ if(ret != 0)
+   goto failure;
+ m->problem_status_lock_ini= 1;
+ ret= pthread_mutex_init(&(m->msg_watcher_lock), NULL);
+ if(ret != 0)
+   goto failure;
+ m->msg_watcher_lock_ini= 1;
+ ret= pthread_mutex_init(&(m->msgw_fetch_lock), NULL);
+ if(ret != 0)
+   goto failure;
+ m->msgw_fetch_lock_ini= 1;
 
  if(leafname != NULL)
    free(leafname);
@@ -480,6 +543,7 @@ int Xorriso_destroy(struct XorrisO **xorriso, int flag)
  Exclusions_destroy(&(m->disk_exclusions), 0);
  Exclusions_destroy(&(m->iso_rr_hidings), 0);
  Exclusions_destroy(&(m->joliet_hidings), 0);
+ Exclusions_destroy(&(m->hfsplus_hidings), 0);
  Xorriso_destroy_all_extf(m, 0);
  Xorriso_lst_destroy_all(&(m->drive_blacklist), 0);
  Xorriso_lst_destroy_all(&(m->drive_greylist), 0);
@@ -501,6 +565,20 @@ int Xorriso_destroy(struct XorrisO **xorriso, int flag)
 
  Xorriso_detach_libraries(m, flag&1);
 
+ if(m->lib_msg_queue_lock_ini)
+   pthread_mutex_destroy(&(m->lib_msg_queue_lock));
+ if(m->result_msglists_lock_ini)
+   pthread_mutex_destroy(&(m->result_msglists_lock));
+ if(m->write_to_channel_lock_ini)
+   pthread_mutex_destroy(&(m->write_to_channel_lock));
+ if(m->problem_status_lock_ini)
+   pthread_mutex_destroy(&(m->problem_status_lock));
+ if(m->msg_watcher_lock_ini)
+   pthread_mutex_destroy(&(m->msg_watcher_lock));
+ if(m->msgw_fetch_lock_ini)
+   pthread_mutex_destroy(&(m->msgw_fetch_lock));
+ Xorriso_sieve_dispose(m, 0);
+ 
  free((char *) m);
  *xorriso= NULL;
  return(1);
