@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2011 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2012 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -544,9 +544,12 @@ revoke_sorting:;
    report_size= sub_size/1024;
    if(report_size*1024<sub_size)
       report_size++;
-   sprintf(xorriso->result_line, "%7.f ",(double) (report_size));
+   if(xorriso->sh_style_result)
+     sprintf(xorriso->result_line, "%-7.f ",(double) (report_size));
+   else
+     sprintf(xorriso->result_line, "%7.f ",(double) (report_size));
    sprintf(xorriso->result_line+strlen(xorriso->result_line), "%s\n",
-           Text_shellsafe(show_path, sfe, 0));
+           Xorriso_esc_filepath(xorriso,show_path, sfe, 0));
    Xorriso_result(xorriso, 0);
  }
 
@@ -573,7 +576,10 @@ ex:;
 
 
 /* @param flag   bit1= add '+' to perms
-                 bit2-3: hidden_state : 0=off, 1=iso_rr, 2=joliet= 3=on
+                 bit2-7: hidden_state :
+                   bit2= hide in ISO/RR
+                   bit3= hide in Joliet
+                   bit4= hide in HFS+
 */
 int Xorriso__mode_to_perms(mode_t st_mode, char perms[11], int flag)
 {
@@ -607,12 +613,15 @@ int Xorriso__mode_to_perms(mode_t st_mode, char perms[11], int flag)
    else
      perms[8]= 'T';
  }
- hidden_state= (flag >> 2) & 3;
+
+ hidden_state= (flag >> 2) & 63;
  if(hidden_state == 1)
    perms[9]= 'I';
  else if(hidden_state == 2)
    perms[9]= 'J';
- else if(hidden_state == 3)
+ else if(hidden_state == 4)
+   perms[9]= 'A';
+ else if(hidden_state)
    perms[9]= 'H';
  if(flag & 2) {
    if(hidden_state)
@@ -626,7 +635,10 @@ int Xorriso__mode_to_perms(mode_t st_mode, char perms[11], int flag)
 
 /* @param flag bit0= recognize Xorriso_IFBOOT as file type
                bit1= add '+' to perms
-               bit2-3: hidden_state : 0=off, 1=iso_rr, 2=joliet= 3=on
+               bit2-7: hidden_state :
+                  bit2= hide in ISO/RR
+                  bit3= hide in Joliet
+                  bit4= hide in HFS+
 */
 int Xorriso_format_ls_l(struct XorrisO *xorriso, struct stat *stbuf, int flag)
 {
@@ -660,7 +672,7 @@ int Xorriso_format_ls_l(struct XorrisO *xorriso, struct stat *stbuf, int flag)
  else
    strcat(rpt, "?");
 
- Xorriso__mode_to_perms(st_mode, perms, flag & (2 | 12));
+ Xorriso__mode_to_perms(st_mode, perms, flag & (2 | 252));
  strcat(rpt, perms);
 
  sprintf(rpt+strlen(rpt)," %3u ",(unsigned int) stbuf->st_nlink);
@@ -766,6 +778,7 @@ ex:;
    free((char *) current);
  }
  Xorriso_free_meM(name);
+ Dirseq_destroy(&dirseq, 0);
  return(ret);
 }
 
@@ -838,7 +851,7 @@ int Xorriso_lsx_filev(struct XorrisO *xorriso, char *wd,
      if(filec>1) {
        strcpy(xorriso->result_line, "\n");
        Xorriso_result(xorriso,0);
-       Text_shellsafe(filev[i], xorriso->result_line, 0);
+       Xorriso_esc_filepath(xorriso,filev[i], xorriso->result_line, 0);
        strcat(xorriso->result_line, ":\n");
        Xorriso_result(xorriso,0);
      }
@@ -884,14 +897,17 @@ int Xorriso_lsx_filev(struct XorrisO *xorriso, char *wd,
        if(ret==0)
  continue;
      }
-     sprintf(rpt, "%7.f ",(double) (size/1024));
+     if(xorriso->sh_style_result)
+       sprintf(rpt, "%-7.f ",(double) (size/1024));
+     else
+       sprintf(rpt, "%7.f ",(double) (size/1024));
    }
    if(link_target[0]) {
-     Text_shellsafe(filev[i], xorriso->result_line, 1);
+     Xorriso_esc_filepath(xorriso,filev[i], xorriso->result_line, 1);
      strcat(xorriso->result_line, " -> ");
-     Text_shellsafe(link_target, xorriso->result_line, 1 | 2);
+     Xorriso_esc_filepath(xorriso,link_target, xorriso->result_line, 1 | 2);
    } else {
-     Text_shellsafe(filev[i], xorriso->result_line, 1);
+     Xorriso_esc_filepath(xorriso,filev[i], xorriso->result_line, 1);
    }
    strcat(xorriso->result_line, "\n");
    Xorriso_result(xorriso, 0);
@@ -1215,7 +1231,7 @@ int Xorriso_findx_action(struct XorrisO *xorriso, struct FindjoB *job,
  } else if(action == 44) { /* list_extattr */
    ret= Xorriso_list_extattr(xorriso, NULL, abs_path, show_path, target, 2);
  } else {
-   Text_shellsafe(show_path, xorriso->result_line, 0);
+   Xorriso_esc_filepath(xorriso,show_path, xorriso->result_line, 0);
    strcat(xorriso->result_line, "\n");
    Xorriso_result(xorriso, 0);
    ret= 1;
@@ -1765,7 +1781,7 @@ int Xorriso_afile_fopen(struct XorrisO *xorriso,
  if(fp==NULL){
    if(!(flag&1)) {
      sprintf(xorriso->info_text,
-             "Failed to open file '%s' in %s mode\n", filename, mode);
+             "Failed to open file '%s' in %s mode", filename, mode);
      Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, errno, "FAILURE", 0);
    }
    return(0);
