@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2012 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2013 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -211,6 +211,40 @@ int Xorriso_is_isohybrid(struct XorrisO *xorriso, IsoFile *bootimg_node,
 }
 
 
+/* @return <0 yes , 0 no , <0 error */
+int Xorriso_is_grub2_elto(struct XorrisO *xorriso, IsoFile *bootimg_node,
+                          int flag)
+{
+#define Xorriso_grub2_boot_info_poS 2548
+ int ret, i;
+ unsigned char buf[Xorriso_grub2_boot_info_poS + 8];
+ void *data_stream= NULL;
+ off_t blk;
+
+ ret= Xorriso_iso_file_open(xorriso, "", (void *) bootimg_node,
+                            &data_stream, 1);
+ if(ret <= 0)
+   return(-1);
+ ret= Xorriso_iso_file_read(xorriso, data_stream, (char *) buf,
+                            Xorriso_grub2_boot_info_poS  + 8, 0);
+ Xorriso_iso_file_close(xorriso, &data_stream, 0);
+ if(ret < Xorriso_grub2_boot_info_poS + 8)
+   return(0);
+
+ /* >>> ??? Check for some id */;
+
+ /* Check whether it has grub2_boot_info patching */
+ blk= 0;
+ for(i= 0; i < 8; i++)
+   blk|= buf[Xorriso_grub2_boot_info_poS + i] << (8 * i);
+ blk-= 5;
+ blk/= 4;
+ if(blk != (off_t) xorriso->loaded_boot_bin_lba)
+   return(0);
+ return(1);
+}
+
+
 int Xorriso_image_has_md5(struct XorrisO *xorriso, int flag)
 {
  int ret;
@@ -243,7 +277,7 @@ int Xorriso_pvd_info(struct XorrisO *xorriso, int flag)
  IsoImage *image;
  struct burn_drive_info *dinfo;
  struct burn_drive *drive;
- char *msg, block_head[8];
+ char *msg, block_head[8], *crt, *mdt, *ext, *eft;
  off_t head_count;
 
  msg= xorriso->result_line;
@@ -292,6 +326,19 @@ int Xorriso_pvd_info(struct XorrisO *xorriso, int flag)
  Xorriso_result(xorriso,0);
  sprintf(msg, "Biblio File  : %s\n", xorriso->biblio_file);
  Xorriso_result(xorriso,0);
+
+ ret= iso_image_get_pvd_times(image, &crt, &mdt, &ext, &eft);
+ if(ret != ISO_SUCCESS)
+   crt= mdt= ext= eft= "";
+ sprintf(msg, "Creation Time: %s\n", crt);
+ Xorriso_result(xorriso,0);
+ sprintf(msg, "Modif. Time  : %s\n", mdt);
+ Xorriso_result(xorriso,0);
+ sprintf(msg, "Expir. Time  : %s\n", ext);
+ Xorriso_result(xorriso,0);
+ sprintf(msg, "Eff. Time    : %s\n", eft);
+ Xorriso_result(xorriso,0);
+
  return(1);
 }
 
@@ -420,7 +467,14 @@ int Xorriso_boot_item_status(struct XorrisO *xorriso, char *cat_path,
 
  is_default= ((patch_isolinux & 1) == 0 || bin_path[0] == 0 || (flag & 4));
  sprintf(line, "-boot_image %s boot_info_table=%s\n",
-         (patch_isolinux & 2) ? "grub" : form, patch_isolinux ? "on" : "off");
+               (patch_isolinux & 2) ? "grub" : form,
+               (patch_isolinux & 1) ? "on" : "off");
+ if(!(is_default && no_defaults))
+   Xorriso_status_result(xorriso,filter,fp,flag&2); 
+ 
+ is_default= ((patch_isolinux & 512) == 0 || bin_path[0] == 0 || (flag & 4));
+ sprintf(line, "-boot_image grub grub2_boot_info=%s\n",
+               (patch_isolinux & 512) ? "on" : "off");
  if(!(is_default && no_defaults))
    Xorriso_status_result(xorriso,filter,fp,flag&2); 
  
@@ -479,6 +533,10 @@ int Xorriso_boot_status_non_mbr(struct XorrisO *xorriso, IsoImage *image,
    Text_shellsafe(xorriso->ascii_disc_label, line, 1);
    strcat(line, "\n");
    Xorriso_status_result(xorriso, filter, fp, flag & 2);
+   sprintf(line, "-boot_image grub grub2_sparc_core=");
+   Text_shellsafe(xorriso->grub2_sparc_core, line, 1);
+   strcat(line, "\n");
+   Xorriso_status_result(xorriso, filter, fp, flag & 2);
    return(0);
  }
  if(sa_type != 1 && sa_type != 2)
@@ -620,7 +678,7 @@ no_image:;
        else
          media_type= 0;
        ret= Xorriso_boot_item_status(xorriso, xorriso->boot_image_cat_path,
-                  path, platform_id, patch & 1, media_type,
+                  path, platform_id, patch, media_type,
                   load_size, id_string, sel_crit, "any",
                   filter, fp, 16 | (flag & 3));
        if(ret <= 0)
@@ -694,6 +752,8 @@ int Xorriso__append_boot_params(char *line, ElToritoBootImage *bootimg,
            " , platform_id=0x%-2.2X ", (unsigned int) platform_id);
  if(el_torito_seems_boot_info_table(bootimg, 0))
    sprintf(line + strlen(line), " , boot_info_table=on");
+ if(el_torito_seems_boot_info_table(bootimg, 1))
+   sprintf(line + strlen(line), " , grub2_boot_info=on");
  return(1);
 }
 
@@ -704,9 +764,10 @@ int Xorriso__append_boot_params(char *line, ElToritoBootImage *bootimg,
 */
 int Xorriso_show_boot_info(struct XorrisO *xorriso, int flag)
 {
- int ret, bin_path_valid= 0,has_isolinux_mbr= 0, i, num_boots;
+ int ret, bin_path_valid= 0,has_isolinux_mbr= 0, i, num_boots, has_mbr= 0;
+ int has_grub2_mbr= 0;
  unsigned int mbr_lba= 0;
- off_t lb0_count;
+ off_t lb0_count, blk;
  char *respt, *path;
  unsigned char *lb0= NULL;
  struct burn_drive_info *dinfo;
@@ -759,18 +820,19 @@ no_boot:;
      bin_path_valid= 1;
  }
  sprintf(respt, "Boot record  : El Torito");
+ /* Load and examine potential MBR */
+ ret= burn_read_data(drive, (off_t) 0, (char *) lb0, (off_t) 2048,
+                     &lb0_count, 2);
+ if(ret > 0)
+   if(lb0[510] == 0x55 && lb0[511] == 0xaa)
+     has_mbr= 1;
  if(bin_path_valid)
    ret= Xorriso_is_isohybrid(xorriso, bootimg_node, 0);
  else
    ret= 0;
  if(ret > 0) {
-   /* Load and examine potential MBR */
-   ret= burn_read_data(drive, (off_t) 0, (char *) lb0, (off_t) 2048,
-                       &lb0_count, 2);
-   if(ret > 0) {
+   if(has_mbr) {
      has_isolinux_mbr= 1;
-     if(lb0[510] != 0x55 || lb0[511] != 0xaa)
-       has_isolinux_mbr= 0;
      mbr_lba= lb0[432] | (lb0[433] << 8) | (lb0[434] << 16) | (lb0[435] << 24);
      mbr_lba/= 4;
      if(mbr_lba != (unsigned int) xorriso->loaded_boot_bin_lba)
@@ -782,17 +844,35 @@ no_boot:;
        if(i >= 426)
          has_isolinux_mbr= 0;
      }
-     for(i= 462; i < 510; i++)
-       if(lb0[i])
-     break;
-     if(i < 510)
-       has_isolinux_mbr= 0;
    }
    if(has_isolinux_mbr)
      strcat(respt, " , ISOLINUX isohybrid MBR pointing to boot image");
    else
      strcat(respt, " , ISOLINUX boot image capable of isohybrid");
  }
+ if(bin_path_valid)
+   ret= Xorriso_is_grub2_elto(xorriso, bootimg_node, 0);
+ else
+   ret= 0;
+ if(ret > 0) {
+   if(has_mbr) {
+     has_grub2_mbr= 1;
+     blk= 0;
+     for(i= 0; i < 8; i++)
+       blk|= lb0[i + 0x1b0] << (8 * i);
+     blk-= 4;
+     blk/= 4;
+     if(blk != (off_t) xorriso->loaded_boot_bin_lba)
+       has_grub2_mbr= 0;
+     for(i= 32; i < 427; i++)
+       if(strncmp((char *) (lb0 + i), "GRUB", 4) == 0)
+     break;
+     if(i >= 427)
+       has_grub2_mbr= 0;
+   }
+   if(has_grub2_mbr)
+     strcat(respt, " , GRUB2 MBR pointing to first boot image");
+ }
  strcat(respt, "\n");
  Xorriso_toc_line(xorriso, flag & 8);
  if(flag & 2)
@@ -1000,6 +1080,10 @@ int Xorriso_mount(struct XorrisO *xorriso, char *dev, int adr_mode,
    aquire_flag= 1;
    if((xorriso->toc_emulation_flag & 2) && adr_mode == 3)
      aquire_flag|= 16;
+   if(xorriso->toc_emulation_flag & 4)
+     aquire_flag|= 128;
+   if(xorriso->toc_emulation_flag & 8)
+     aquire_flag|= 512;
    ret= isoburn_drive_aquire(&dinfo, libburn_adr, aquire_flag);
    burn_preset_device_open(1, 0, 0);
    Xorriso_process_msg_queues(xorriso,0);
@@ -1151,7 +1235,7 @@ int Xorriso_coordinate_system_area(struct XorrisO *xorriso, int sa_type,
       "SUN Disk Label"};
 
  old_type= (xorriso->system_area_options & 0xfc) >> 2;
- old_options= xorriso->system_area_options & ~0xfc;
+ old_options= xorriso->system_area_options & ~0x40fc;
  if((old_type != 0 || old_options != 0) &&
     (old_type != sa_type || (old_options != 0 && old_options != options))) {
    sprintf(xorriso->info_text, "%s : First sector already occupied by %s",
@@ -1163,7 +1247,8 @@ int Xorriso_coordinate_system_area(struct XorrisO *xorriso, int sa_type,
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
    goto hint_revoke;
  }
- xorriso->system_area_options= ((sa_type << 2) & 0xfc) | (options & ~0xfc);
+ xorriso->system_area_options= (xorriso->system_area_options & 0x4000) |
+                               ((sa_type << 2) & 0xfc) | (options & ~0x40fc);
  if(sa_type == 0)
    xorriso->patch_system_area= xorriso->system_area_options;
  return(1);
