@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2011 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2012 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -98,6 +98,9 @@ int Xorriso_normalize_img_path(struct XorrisO *xorriso, char *wd,
    {ret= 2; goto ex;} /* root directory */
  }
 
+ if(apt[0] == '.')
+   if(apt[1] == 0 || apt[1] == '/')
+     is_dir= 1;
  for(npt= apt; !done; apt= npt+1) {
    npt= strchr(apt, '/');
    if(npt==NULL) {
@@ -112,10 +115,8 @@ int Xorriso_normalize_img_path(struct XorrisO *xorriso, char *wd,
  break;
  continue;
    }
-   if(strcmp(apt,".")==0) {
-     is_dir= 1;
+   if(strcmp(apt,".")==0)
  continue;
-   }
    if(strcmp(apt,"..")==0) {
      if(!(flag&2)) {
        node= (IsoNode *) dir;
@@ -137,6 +138,7 @@ bonked_root:;
      is_dir= 1;
  continue;
    }
+   is_dir= 0;
    ret= Sfile_add_to_path(eff_path, apt, 0);
    if(ret<=0) {
 much_too_long:;
@@ -163,6 +165,8 @@ much_too_long:;
      {ret= 0; goto ex;}
    if(ret==2)
      is_dir= 1;
+   else
+     is_dir= 0;
  }
  ret= 1+!!is_dir;
 ex:;
@@ -1444,9 +1448,12 @@ much_too_long:;
    report_size= sub_size/1024;
    if(report_size*1024<sub_size)
       report_size++;
-   sprintf(xorriso->result_line, "%7.f ",(double) (report_size));
+   if(xorriso->sh_style_result)
+     sprintf(xorriso->result_line, "%-7.f ",(double) (report_size));
+   else
+     sprintf(xorriso->result_line, "%7.f ",(double) (report_size));
    sprintf(xorriso->result_line+strlen(xorriso->result_line), "%s\n",
-           Text_shellsafe(show_path, sfe, 0));
+           Xorriso_esc_filepath(xorriso, show_path, sfe, 0));
    Xorriso_result(xorriso, 0);
  }
  ret= 1;
@@ -1533,12 +1540,14 @@ int Xorriso_node_eff_hidden(struct XorrisO *xorriso, IsoNode *node, int flag)
  IsoNode *current, *parent;
 
  current= node;
- for(current= node; hidden_state != 3;) {
+ for(current= node; hidden_state != 7;) {
    ret= iso_node_get_hidden(current);
    if(ret & LIBISO_HIDE_ON_RR)
      hidden_state|= 1;
    if(ret & LIBISO_HIDE_ON_JOLIET)
      hidden_state|= 2;
+   if(ret & LIBISO_HIDE_ON_HFSPLUS)
+     hidden_state|= 4;
    parent= (IsoNode *) iso_node_get_parent(current);
    if(parent == current)
  break;
@@ -1615,7 +1624,7 @@ int Xorriso_ls_filev(struct XorrisO *xorriso, char *wd,
      if(filec>1) {
        strcpy(xorriso->result_line, "\n");
        Xorriso_result(xorriso,0);
-       Text_shellsafe(filev[i], xorriso->result_line, 0);
+       Xorriso_esc_filepath(xorriso, filev[i], xorriso->result_line, 0);
        strcat(xorriso->result_line, ":\n");
        Xorriso_result(xorriso,0);
      }
@@ -1664,14 +1673,17 @@ int Xorriso_ls_filev(struct XorrisO *xorriso, char *wd,
        if(ret==0)
  continue;
      }
-     sprintf(rpt, "%7.f ",(double) (size/1024));
+     if(xorriso->sh_style_result)
+       sprintf(rpt, "%-7.f ",(double) (size/1024));
+     else
+       sprintf(rpt, "%7.f ",(double) (size/1024));
    }
    if(link_target[0] && (flag&5)==1) {
-     Text_shellsafe(filev[i], xorriso->result_line, 1),
+     Xorriso_esc_filepath(xorriso, filev[i], xorriso->result_line, 1),
      strcat(xorriso->result_line, " -> ");
-     Text_shellsafe(link_target, xorriso->result_line, 1 | 2);
+     Xorriso_esc_filepath(xorriso, link_target, xorriso->result_line, 1 | 2);
    } else {
-     Text_shellsafe(filev[i], xorriso->result_line, 1);
+     Xorriso_esc_filepath(xorriso, filev[i], xorriso->result_line, 1);
    }
    strcat(xorriso->result_line, "\n");
    Xorriso_result(xorriso, 0);
@@ -1777,11 +1789,11 @@ cannot_create_iter:;
  continue;
    }
    if(link_target[0] && (flag&1)) {
-     Text_shellsafe(npt, xorriso->result_line, 1); 
+     Xorriso_esc_filepath(xorriso, npt, xorriso->result_line, 1); 
      strcat(xorriso->result_line, " -> ");
-     Text_shellsafe(link_target, xorriso->result_line, 1 | 2);
+     Xorriso_esc_filepath(xorriso, link_target, xorriso->result_line, 1 | 2);
    } else {
-     Text_shellsafe(npt, xorriso->result_line, 1);
+     Xorriso_esc_filepath(xorriso, npt, xorriso->result_line, 1);
    }
    strcat(xorriso->result_line, "\n");
    Xorriso_result(xorriso, 0);
@@ -1877,6 +1889,7 @@ ex:;
 
 /* @param flag bit0= a match count !=1 is a FAILURE event
                bit1= with bit0 tolerate 0 matches if pattern is a constant
+               bit2= do not issue debug messages about temporay memory needs
 */
 int Xorriso_expand_pattern(struct XorrisO *xorriso,
                            int num_patterns, char **patterns, int extra_filec,
@@ -1959,7 +1972,7 @@ int Xorriso_expand_pattern(struct XorrisO *xorriso,
  mem+= extra_filec*sizeof(char *);
  if(count<=0)
    {ret= 0; goto ex;}
- ret= Xorriso_alloc_pattern_mem(xorriso, *mem, count, filev, 0);
+ ret= Xorriso_alloc_pattern_mem(xorriso, *mem, count, filev, !!(flag & 4));
  if(ret<=0)
    goto ex;
  /* now store addresses */
@@ -2067,12 +2080,78 @@ int Xorriso__file_start_lba(IsoNode *node,
 }
 
 
+/* flag bit0= examine sub directories rather than data files
+*/
+int Xorriso_dir_disk_path(struct XorrisO *xorriso, IsoNode *dir_node,
+                          char disk_path[SfileadrL], int flag)
+{
+ int ret;
+ char *npt;
+ IsoNode *node;
+ IsoDir *dir;
+ IsoDirIter *iter= NULL;
+
+ dir= (IsoDir *) dir_node;
+ ret= iso_dir_get_children(dir, &iter);
+ if(ret<0) {
+   Xorriso_cannot_create_iter(xorriso, ret, 0);
+   {ret= -1; goto ex;}
+ }
+ while(1) {
+   ret= iso_dir_iter_next(iter, &node);
+   if(ret < 0) {
+     Xorriso_report_iso_error(xorriso, "", ret,
+               "Error when iterating over directory", 0, "FAILURE", 1);
+     ret= -1; goto ex;
+   }
+   if(ret == 0)
+ break;
+     
+   if(LIBISO_ISDIR(node) && (flag & 1)) {
+     ret= Xorriso_dir_disk_path(xorriso, node, disk_path, flag);
+     if(ret < 0)
+       goto ex;
+     if(ret == 0)
+ continue;
+   } else if(LIBISO_ISREG(node) && !(flag & 1)) {
+     ret= Xorriso_retrieve_disk_path(xorriso, node, disk_path, 0);
+     if(ret < 0)
+       goto ex;
+     if(ret == 0)
+ continue;
+   } else
+ continue;
+   /* Use its parent dir as answer */
+   npt= strrchr(disk_path, '/');
+   if(npt == NULL || npt == disk_path)
+     strcpy(disk_path, "/");
+   else
+     *npt= 0;
+   ret= 1; goto ex;
+ }
+ if(!(flag & 1))
+   ret= Xorriso_dir_disk_path(xorriso, dir_node, disk_path, 1);
+ else
+   ret= 0;
+ex:
+ if(iter != NULL)
+   iso_dir_iter_free(iter);
+ return(ret);
+}
+
+
 int Xorriso_retrieve_disk_path(struct XorrisO *xorriso, IsoNode *node,
                                char disk_path[SfileadrL], int flag)
 {
  IsoFile *file;
  IsoStream *stream= NULL, *input_stream;
  char type_text[80], *source_path = NULL;
+ int ret;
+
+ if(LIBISO_ISDIR(node)) {
+   ret= Xorriso_dir_disk_path(xorriso, node, disk_path, 0);
+   return(ret);
+ }
 
  if(!LIBISO_ISREG(node)) 
    return(0);
