@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2011 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2013 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -30,6 +30,28 @@
 #include "xorrisoburn.h"
 
 
+/* Command -data_cache_size */
+int Xorriso_option_data_cache_size(struct XorrisO *xorriso, char *num_tiles,
+                                   char *tile_blocks, int flag)
+{
+ int ret, blocks= -1, tiles= -1, to_default= 0;
+
+ sscanf(num_tiles, "%d", &tiles);
+ sscanf(tile_blocks, "%d", &blocks);
+ if(strcmp(num_tiles, "default") == 0 || num_tiles[0] == 0)
+   to_default|= 1;
+ if(strcmp(tile_blocks, "default") == 0 || tile_blocks[0] == 0)
+   to_default|= 2;
+ ret= Xorriso_set_data_cache(xorriso, NULL, tiles, blocks, to_default);
+ if(ret > 0) {
+   xorriso->cache_num_tiles= tiles;
+   xorriso->cache_tile_blocks= blocks;
+   xorriso->cache_default= to_default;
+ }
+ return(ret);
+}
+
+
 /* Options -dev , -indev, -outdev */
 /** @param flag bit0= use as indev
                 bit1= use as outdev
@@ -60,7 +82,7 @@ int Xorriso_option_dev(struct XorrisO *xorriso, char *in_adr, int flag)
    }
  }
 
- if(xorriso->volset_change_pending && (flag&1)) {
+ if(Xorriso_change_is_pending(xorriso, 0) && (flag&1)) {
    sprintf(xorriso->info_text,
            "%s: Image changes pending. -commit or -rollback first",
            (flag&2) ? "-dev" : "-indev");
@@ -109,7 +131,7 @@ int Xorriso_option_devices(struct XorrisO *xorriso, int flag)
 {
  int ret;
 
- if(xorriso->volset_change_pending) {
+ if(Xorriso_change_is_pending(xorriso, 0)) {
    sprintf(xorriso->info_text,
            "-devices: Image changes pending. -commit or -rollback first");
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
@@ -150,7 +172,7 @@ int Xorriso_option_dialog(struct XorrisO *xorriso, char *mode, int flag)
  if(strcmp(mode, "on") == 0 || strcmp(mode, "multi_line") == 0)
    xorriso->dialog= 2;
  else if(strcmp(mode, "single_line") == 0)
-   xorriso->dialog= 2;
+   xorriso->dialog= 1;
  else if(strcmp(mode, "off") == 0)
    xorriso->dialog= 0;
  else {
@@ -331,7 +353,7 @@ int Xorriso_option_eject(struct XorrisO *xorriso, char *which, int flag)
    gu_flag|= 2;
  else
    gu_flag|= 3;
- if((gu_flag&1) && xorriso->volset_change_pending) {
+ if((gu_flag&1) && Xorriso_change_is_pending(xorriso, 0)) {
    sprintf(xorriso->info_text,
            "-eject: Image changes pending. -commit or -rollback first");
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
@@ -358,8 +380,8 @@ int Xorriso_option_end(struct XorrisO *xorriso, int flag)
    cmd= "-rollback_end";
  else
    cmd= "-end";
- if(xorriso->volset_change_pending) {
-   if((flag & 1) || xorriso->volset_change_pending != 1)
+ if(Xorriso_change_is_pending(xorriso, 0)) {
+   if((flag & 1) || !Xorriso_change_is_pending(xorriso, 1))
      which_will= "end the program discarding image changes";
    else
      which_will= "commit image changes and then end the program";
@@ -372,8 +394,8 @@ int Xorriso_option_end(struct XorrisO *xorriso, int flag)
      return(2);
  }
 
- if(xorriso->volset_change_pending) {
-   if((flag & 1) || xorriso->volset_change_pending != 1) {
+ if(Xorriso_change_is_pending(xorriso, 0)) {
+   if((flag & 1) || !Xorriso_change_is_pending(xorriso, 1)) {
      xorriso->volset_change_pending= 0;
    } else {
      ret= Xorriso_option_commit(xorriso, 1);
@@ -484,7 +506,7 @@ int Xorriso_option_external_filter(struct XorrisO *xorriso,
  (*idx)= end_idx;
  if(end_idx - start_idx < 3) {
    sprintf(xorriso->info_text,
- "-external_filter : Not enough arguments given. Needed: name options path %s",
+"-external_filter : Not enough parameters given. Needed: name options path %s",
            xorriso->list_delimiter);
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
    return(0);
@@ -543,7 +565,7 @@ int Xorriso_option_extract(struct XorrisO *xorriso, char *iso_path,
 
  if(!(flag&2))
    Xorriso_pacifier_callback(xorriso, "files restored",xorriso->pacifier_count,
-                             xorriso->pacifier_total, "", 1|4);
+                             xorriso->pacifier_total, "", 1 | 4 | 8 | 32);
  if(ret <= 0 || problem_count > 0)
    goto ex;
 
@@ -645,15 +667,17 @@ ex:;
                bit2= do not count deleted files with rm and rm_r
                bit3= use Xorriso_findi_sorted() rather than Xorriso_findi()
                      (this can also be ordered by test -sort_lba)
+               bit4= return number of matches plus 1
 */
 int Xorriso_option_find(struct XorrisO *xorriso, int argc, char **argv,
                        int *idx, int flag)
 {
  int ret, i, end_idx, type= 0, action, deleter= 0, start_lba, count;
- int list_extattr_head= 0, bsl_mem;
+ int list_extattr_head= 0, bsl_mem, disk_path;
  struct FindjoB *job, *first_job= NULL, *new_job;
  char *start_path, *path= NULL, *cpt, *other_path_start= NULL, *cd_pt;
  char *access_acl_text= NULL, *default_acl_text= NULL, *list_extattr_mode;
+ char *arg1_pt;
 
  struct stat dir_stbuf;
  uid_t user= 0;
@@ -683,7 +707,7 @@ int Xorriso_option_find(struct XorrisO *xorriso, int argc, char **argv,
      if(i+1>=end_idx) {
 not_enough_arguments:;
        sprintf(xorriso->info_text,
-               "-find[ix]: not enough arguments with test ");
+               "-find[ix]: not enough parameters with test ");
        Text_shellsafe(argv[i], xorriso->info_text, 1);
        goto sorry_ex;
      }
@@ -747,14 +771,25 @@ not_enough_arguments:;
      Findjob_set_filter_filter(job, -1, 0);
    } else if(strcmp(argv[i], "-has_md5")==0) {
      Findjob_set_prop_filter(job, 15, 1, 0);
-   } else if(strcmp(argv[i], "-disk_name")==0) {
+   } else if(strcmp(argv[i], "-disk_name")==0 ||
+             strcmp(argv[i], "-disk_path")==0) {
+     disk_path= (strcmp(argv[i], "-disk_path") == 0);
      if(i+1>=end_idx)
        goto not_enough_arguments;
      i++;
-     ret= Findjob_set_name_expr(job, argv[i], 2);
+     arg1_pt= argv[i];
+     if(disk_path) {
+       ret= Xorriso_make_abs_adr(xorriso, xorriso->wdx, argv[i], path,
+                                1 | 2 | 4 | 8);
+       if(ret<=0)
+         goto ex;
+       arg1_pt= path;
+     }
+     ret= Findjob_set_name_expr(job, arg1_pt, 2 + disk_path);
      if(ret<=0) {
        sprintf(xorriso->info_text,
-               "-find[ix]: cannot set -disk_name expression ");
+               "-find[ix]: cannot set %s ",
+               disk_path ? "-disk_path address" : "-disk_name expression");
        Text_shellsafe(argv[i], xorriso->info_text, 1);
        goto sorry_ex;
      }
@@ -774,6 +809,28 @@ not_enough_arguments:;
          goto sorry_ex;
        }
      }
+   } else if(strcmp(argv[i], "-has_hfs_crtp")==0) {
+     if(i + 2 >= end_idx)
+       goto not_enough_arguments;
+     i+= 2;
+     ret= Xorriso_hfsplus_file_creator_type(xorriso, "", NULL,
+                                            argv[i - 1], argv[i], 3);
+     if(ret <= 0)
+       {ret= 0; goto ex;}
+     ret= Findjob_set_crtp_filter(job, argv[i - 1], argv[i], 0);
+     if(ret <= 0) {
+       sprintf(xorriso->info_text, "-findi: cannot setup -has_hfs_crtp test");
+       goto sorry_ex;
+     }
+   } else if(strcmp(argv[i], "-has_hfs_bless")==0) {
+     if(i + 1 >= end_idx)
+       goto not_enough_arguments;
+     i+= 1;
+     ret= Findjob_set_bless_filter(xorriso, job, argv[i], 0);
+     if(ret <= 0) {
+       sprintf(xorriso->info_text, "-findi: cannot setup -has_hfs_bless test");
+       goto sorry_ex;
+     }
    } else if(strcmp(argv[i], "-true") == 0) {
      ret= Findjob_set_false(job, -1, 0);
    } else if(strcmp(argv[i], "-false") == 0) {
@@ -815,7 +872,7 @@ not_enough_arguments:;
      if(i+1>=end_idx) {
 not_enough_exec_arguments:;
        sprintf(xorriso->info_text,
-               "-find[ix]: not enough arguments with -exec ");
+               "-find[ix]: not enough parameters with -exec ");
        Text_shellsafe(argv[i], xorriso->info_text, 1);
        goto sorry_ex;
      }
@@ -1059,6 +1116,29 @@ not_enough_exec_arguments:;
        Findjob_set_action_target(job, 44, argv[i], 0);
        list_extattr_head= 1;
        list_extattr_mode= argv[i];
+     } else if(strcmp(cpt, "set_hfs_crtp")==0) {
+       if(i + 2 >= end_idx)
+         goto not_enough_exec_arguments;
+       i+= 2;
+       /* Check creator and type for compliance */
+       ret= Xorriso_hfsplus_file_creator_type(xorriso, "", NULL,
+                                              argv[i - 1], argv[i], 1);
+       if(ret <= 0)
+         goto ex;
+       Findjob_set_action_text_2(job, 45, argv[i - 1], argv[i], 0);
+     } else if(strcmp(cpt, "get_hfs_crtp")==0) {
+       Findjob_set_action_target(job, 46, NULL, 0);
+     } else if(strcmp(cpt, "set_hfs_bless")==0) {
+       if(i+1>=end_idx)
+         goto not_enough_exec_arguments;
+       i++;
+       /* Check type of blessing for compliance */
+       ret= Xorriso_hfsplus_bless(xorriso, "", NULL, argv[i], 4);
+       if(ret <= 0)
+         goto ex;
+       Findjob_set_action_target(job, 47, argv[i], 0);
+     } else if(strcmp(cpt, "get_hfs_bless")==0) {
+       Findjob_set_action_target(job, 48, NULL, 0);
      } else {
        sprintf(xorriso->info_text, "-find -exec: unknown action ");
        Text_shellsafe(argv[i], xorriso->info_text, 1);
@@ -1117,7 +1197,7 @@ ex:;
                              xorriso->pacifier_count, 0, "", 1 | 2);
  else if(mem_lut!=xorriso->last_update_time && mem_lut!=0.0 && !(flag&2))
    Xorriso_pacifier_callback(xorriso, "content bytes read",
-                             xorriso->pacifier_count, 0, "", 1);
+                             xorriso->pacifier_count, 0, "", 1 | 8 | 32);
  if(first_job->action == 35 && !(flag & 1))
    Xorriso_report_md5_outcome(xorriso, first_job->target, 0);
  if(first_job->action == 40) {
@@ -1133,6 +1213,8 @@ ex:;
    free(access_acl_text);
  if(default_acl_text != NULL)
    free(default_acl_text);
+ if(ret > 0 && (flag & 16) && first_job != NULL)
+   ret= first_job->match_count + 1;
  Findjob_destroy(&first_job, 0);
  Xorriso_free_meM(path);
  Xorriso_free_meM(other_path_start);
@@ -1381,6 +1463,10 @@ int Xorriso_option_hardlinks(struct XorrisO *xorriso, char *mode, int flag)
      xorriso->ino_behavior&= ~16;
    } else if(strcmp(what, "cheap_sorted_extract") == 0) {
      xorriso->ino_behavior|= 16;
+   } else if(strcmp(what, "lsl_count") == 0) {
+     xorriso->ino_behavior&= ~32;
+   } else if(strcmp(what, "no_lsl_count") == 0) {
+     xorriso->ino_behavior|= 32;
    } else {
      sprintf(xorriso->info_text, "-hardlinks: unknown mode '%s' in '%s'",
              what, mode);
@@ -1422,9 +1508,18 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "This program creates, loads, manipulates and writes ISO 9660 filesystem",
 "images with Rock Ridge extensions. Write targets can be drives with optical",
 "media or local filesystem objects.",
+"The program operations are controlled by a sequence of commands, of which",
+"the initial ones are given as program arguments or as lines in startup",
+"files. Further commands may get read from files in batch mode or from",
+"standard input in dialog mode.",
 "",
-"Preparation options:",
-"Drive addresses are either /dev/... as listed with option -devices or",
+"  -x             Only in effect if given as program argument:",
+"                 Execute commands given as program arguments in a sequence",
+"                 that most likely makes some sense. Default is to execute",
+"                 program arguments exactly in the sequence as given.",
+"",
+"Preparation commands:",
+"Drive addresses are either /dev/... as listed with command -devices or",
 "disk files, eventually with prefix \"stdio:\" if non-CD-drive in /dev tree.",
 "E.g. /dev/sr0 , /tmp/pseudo_drive , stdio:/dev/sdc",
 "  -dev address   Set input and output drive and load eventual ISO image.",
@@ -1446,6 +1541,7 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 displacement versus the start address for which the image",
 "                 was prepared.",
 "  -rom_toc_scan \"on\"|\"force\"|\"off\"[:\"emul_on\"|\"emul_off\"]",
+"                [:\"emul_wide\"|\"emul_narrow\"]",
 "                 Enable scanning for ISO sessions on read-only drives/media",
 "                 resp. on overwriteable media with emulated TOC.",
 "  -calm_drive \"in\"|\"out\"|\"all\"|\"on\"|\"off\"",
@@ -1484,8 +1580,11 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 Allow for writing only the usage of optical drives.",
 "  -early_stdio_test \"on\"|\"appendable_wo\"|\"off\"",
 "                 Classify stdio drives by effective access permissions.",
-"  -blank \"fast\"|\"all\"|\"deformat\"|\"deformat_quickest\"",
+"  -data_cache_size number_of_tiles blocks_per_tile",
+"                 Adjust size and granularity of the data read cache.",
+"  -blank [\"force:\"]\"fast\"|\"all\"|\"deformat\"|\"deformat_quickest\"",
 "                 Blank medium resp. invalidate ISO image on medium.",
+"                 Prefix \"force:\" overrides medium evaluation.",
 "  -close_damaged \"as_needed\"|\"force\"",
 "                 Close track and session of damaged medium.",
 "  -format \"as_needed\"|\"full\"|\"fast\"|\"by_index_#\"|\"by_size_#\"",
@@ -1508,8 +1607,16 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 Specifies the name of the Bibliographic File. (37 chars)",
 "  -abstract_file name",
 "                 Specifies the name of the Abstract File. (37 chars)",
+"  -application_use character|0xXY|disk_path",
+"                 Specifies the content of Application Use field. (512 bytes)",
+"                 A single character or a hex code gets repeated 512 times.",
+"                 Other text gets opened as data file and 512 bytes are read.",
 "  -joliet \"on\"|\"off\"",
 "                 Generate Joliet info additional to Rock Ridge info.",
+"  -hfsplus \"on\"|\"off\"",
+"                 Generate a HFS+ partition and filesystem within ISO image.",
+"  -rockridge \"on\"|\"off\"",
+"                 Opportunity to omit Rock Ridge info. (Do not do it !)",
 "  -jigdo \"clear\"|\"template_path\"|\"jigdo_path\"|\"md5_path\"",
 "         |\"min_size\"|\"checksum_iso\"|\"checksum_template\"",
 "         |\"compression\"|\"exclude\"|\"demand_md5\"|\"mapping\"",
@@ -1520,13 +1627,22 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "  -compliance rule[:rule...]",
 "                 Allow more or less harmless deviations from strict standards",
 "                 compliance.",
+"  -rr_reloc_dir name",
+"                 Specifies name of relocation directory in root directory,",
+"                 to which deep subtrees will get relocated if -compliance",
+"                 is set to \"deep_paths_off\".",
 "  -boot_image \"any\"|\"isolinux\"|\"grub\"",
 "              |\"discard\"|\"keep\"|\"patch\"|\"dir=\"|\"bin_path=\"",
 "              |\"cat_path=\"|\"cat_hidden=on|iso_rr|joliet|off\"",
-"              |\"load_size=\"|\"system_area=\"|\"partition_table=on|off\"",
+"              |\"load_size=\"|\"boot_info_table=\"",
+"              |\"grub2_boot_info=\"|\"grub2_mbr=\"",
+"              |\"system_area=\"|\"partition_table=on|off\"",
+"              |\"chrp_boot_part=on|off=\"|\"prep_boot_part=\"",
+"              |\"efi_boot_part=\"|\"efi_boot_part=--efi-boot-image\"",
 "              |\"mips_path=\"|\"mipsel_path=\"|\"mips_discard\"",
-"              |\"sparc_label=\"|\"sparc_discard\"",
-"              |\"show_status\"",
+"              |\"sparc_label=\"|\"grub2_sparc_core=\"|\"sparc_discard\"",
+"              |\"hfsplus_serial=\"|\"hfsplus_block_size=\"",
+"              |\"apm_block_size=\"|\"show_status\"",
 "                 Whether to discard or keep an exiting El Torito boot image.",
 "                 ISOLINUX can be made bootable by dir=/ or dir=/isolinux",
 "                 or dir=/boot/isolinux. Others, like GRUB, by bin_path=...",
@@ -1535,6 +1651,8 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 ISO image by the usual commands like -map or -add.",
 "                 system_area= and partition_table= are for MBR based booting",
 "                 from USB stick. The system_area= file needs not to be added.",
+"                 chrp_boot_part= and prep_boot_part= are for PowerPC.",
+"                 efi_boot_part= is for booting EFI systems from USB stick.",
 "                 mips_path= adds Big Endian MIPS boot files. mipsel_path=",
 "                 sets one Little Endian MIPS boot file. sparc_label=",
 "                 activates SUN Disk Label. All three are mutually exclusive",
@@ -1550,7 +1668,9 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "  -device_links  Like devices, but showing link paths which are hopefully",
 "                 persistent over reboot on modern Linux systems.",
 "",
-"  -toc           Show media specific table of content (sessions).",
+"  -toc           Show media specific tables of content (sessions).",
+"  -toc_of \"in\"|\"out\"|\"all\"[\":short\"]",
+"                 Show -toc of either input drive or output drive or both.",
 "",
 "  -mount_cmd drive entity id path",
 "                 Print to result channel a command suitable to mount the",
@@ -1572,23 +1692,23 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 Print foreseeable available space on output medium",
 "  -pvd_info      Print various id strings of the loaded ISO image."
 "",
-"Options with variable length path list [...] need the list delimiter text",
-"as end mark if they are followed by another option. By default this delimiter",
-"is \"--\". In dialog and with options read from files, the line end serves",
-"as such a mark. With program arguments this mark can be omitted only with",
-"the last option in the list of arguments.",
+"Commands with variable length path list [...] need the list delimiter text",
+"as end mark if they are followed by another command. By default this",
+"delimiter is \"--\". In dialog and with commands read from files, the line",
+"end serves as such a mark. With program arguments this mark can be omitted",
+"only with the last command in the list of program arguments.",
 "For brevity the list delimiter is referred as \"--\" throughout this text.",
 "",
 "  -list_delimiter text   Set the list delimiter to be used instead of \"--\"",
 "                 It has to be a single word, must not be empty, not longer",
 "                 than 80 characters, may mot contain quotation marks.",
 "",
-"Manipulation options:",
+"Manipulation commands:",
 "disk_path is a path to an object in the local filesystem tree.",
 "iso_rr_path is the Rock Ridge name of a file object in the ISO image.",
 "pathspec is either a disk_path or (if allowed) a pair: iso_rr_path=disk_path",
-"Options marked by [***] have variable argument length and perform pattern",
-"expansion if enabled by -iso_rr_pattern resp. -disk_pattern.",
+"Commands marked by [***] have variable length parameter lists and perform",
+"pattern expansion if enabled by -iso_rr_pattern resp. -disk_pattern.",
 "",
 "  -pathspecs \"on\"|\"off\"  Allow or disallow pathspecs of form ",
 "                 iso_rr_path=disk_path . Only \"off\" allows eventual",
@@ -1628,9 +1748,12 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 Delete the given files from the ISO image.",
 "  -rm_r iso_rr_path [***]",
 "                 Delete the given directory trees from ISO image.",
+"  -move iso_rr_path iso_rr_path",
+"                 Rename the single file given by the first iso_rr_path to",
+"                 the second iso_rr_path.",
 "  -mv iso_rr_path [***] iso_rr_path",
-"                 Rename the given file objects in the ISO tree to the last",
-"                 argument in the list.",
+"                 Like shell command mv rename the given file objects in the",
+"                 ISO tree to the last of the iso_rr_path parameters.",
 "  -chown uid iso_rr_path [***]",
 "                 Equivalent to chown in the ISO image.",
 "  -chown_r uid iso_rr_path [***]",
@@ -1666,8 +1789,8 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 access time, modification time, both times.",
 "  -alter_date_r type timestring iso_rr_path [***]",
 "                 Like -alter_date but affecting all files below directories.",
-"  -hide on|iso_rr|joliet|off iso_rr_path [***]",
-"                 Keep names of files out of directory tree, but store their",
+"  -hide on|iso_rr:joliet:hfsplus|off iso_rr_path [***]",
+"                 Keep names of files out of directory trees, but store their",
 "                 data content in the image.",
 "  -find iso_rr_path [test [op] [test ...]] [-exec action [params]]",
 "                 performs an action on files below the given directory in",
@@ -1676,6 +1799,7 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                   -type b|c|d|p|f|l|s|e, -pending_data, -hidden,",
 "                   -lba_range start count, -damaged, -has_acl, -has_xattr,",
 "                   -has_aaip, -has_filter, -has_md5, -has_any_xattr,",
+"                   -has_hfs_crtp, -has_hfs_bless,",
 "                   -prune, -decision yes|no, -true, -false",
 "                 Operators: -not, -or, -and, -sub, (, -subend, ),",
 "                   -if, -then, -elseif, -else, -endif",
@@ -1684,10 +1808,13 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                   rm, rm_r, compare, update, report_damage, report_lba,",
 "                   getfacl, setfacl, getfattr, setfattr, get_any_xattr,",
 "                   list_extattr, get_md5, check_md5, make_md5,",
+"                   set_hfs_crtp, get_hfs_crtp, set_hfs_bless, get_hfs_bless,",
 "                   set_filter, show_stream, mkisofs_r, hide, find.",
-"                 params are their arguments except iso_rr_path.",
+"                 params are their parameters except iso_rr_path.",
 "  -mkdir iso_rr_path [...]",
 "                 Create empty directories if they do not exist yet.",
+"  -lns target_text iso_rr_path",
+"                 Create a symbolic link pointing to target_text",
 "  -rmdir iso_rr_path [***]",
 "                 Delete empty directories.",
 "  -clone iso_rr_path_original iso_rr_path_copy",
@@ -1695,8 +1822,8 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "  -cp_clone iso_rr_path_original [***] iso_rr_path_dest",
 "                 Create ISO to ISO copies according to the rules of cp -r.",
 "",
-"  --             Default list delimiter marking the end of action argument",
-"                 list. It may be changed by option -list_delimiter.",
+"  --             Default list delimiter marking the end of command parameter",
+"                 lists. It may be changed by command -list_delimiter.",
 "",
 "  -not_paths disk_path [***]",
 "                 Add the given paths to the list of excluded absolute paths.",
@@ -1719,7 +1846,7 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 If \"on\" then ask the user for \"y\" or \"n\" with any",
 "                 file before deleting or overwriting it in the ISO image.",
 "",
-"Filter options:",
+"Filter commands:",
 "External filter processes may produce synthetic file content by reading the",
 "original content from stdin and writing to stdout whatever they want.",
 
@@ -1755,15 +1882,21 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 Set global zisofs parameters:",
 "                   level=0|...|9 , block_size=32k|64k|128k , by_magic=on|off",
 "",
-"Write-to-media options:",
+"Write-to-media commands:",
 "  -rollback      Discard the manipulated ISO image and reload it.",
 "",
-"  -commit        Perform the write operation and then perform -dev outdrive.",
+"  -changes_pending \"no\"|\"yes\"|\"mkisofs_printed\"|\"show_status\"",
+"                 Override the automatically determined change status of the",
+"                 loaded image, or show the current status.",
+"  -commit        Perform the write operation if changes are pending.",
+"                 Then perform -dev outdrive.",
 "                 Hint: To perform a final write operation with no new -dev",
-"                       and no new loading of image, execute option -end.",
+"                       and no new loading of image, execute command -end.",
 "  -commit_eject  \"in\"|\"out\"|\"all\"|\"none\"",
 "                 Like -commit but rather eject than load image from outdrive.",
 "                 Give up any unejected drive afterwards.",
+"  -write_type \"auto\"|\"tao\"|\"sao/dao\"",
+"                 Set write type for CD-R[W], DVD-R[W], DVD+R, BD-R.",
 "  -close \"on\"|\"off\"",
 "                 If \"on\" then mark the written medium as not appendable.",
 "  -padding number[\"k\"|\"m\"]|\"included\"|\"appended\"",
@@ -1785,7 +1918,7 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 Immediately eject the medium in -indev, resp. -outdev,",
 "                 resp. both.",
 "",
-"Navigation options:",
+"Navigation commands:",
 "",
 "  -cd iso_rr_path  Change working directory in the ISO image. iso_rr_paths",
 "                 which do not begin with '/' will be inserted beginning at",
@@ -1853,8 +1986,8 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "  -show_stream_r iso_rr_path [***]",
 "                 Like -show_stream but affecting all files below directories.",
 "",
-"Restore options which copy file objects from ISO image to disk filesystem:",
-"  -osirrox \"on\"|\"device_files\"|\"off\"|\"banned\"",
+"Restore commands which copy file objects from ISO image to disk filesystem:",
+"  -osirrox \"on\"|\"device_files\"|\"off\"|\"blocked\"|\"unblock\"|\"banned\"",
 "           [:\"concat_split_on\"|\"concat_split_off\"]",
 "           [:\"auto_chmod_on\"|\"auto_chmod_off\"]",
 "           [:\"sort_lba_on\"|\"sort_lba_off\"]",
@@ -1862,7 +1995,8 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 By default \"off\" the inverse operation of xorriso from ISO",
 "                 image to disk filesystem is disabled. \"on\" allows xorriso",
 "                 to create, overwrite, delete files in the disk filesystem.", 
-"                 \"banned\" is irrevocably \"off\".",
+"                 \"banned\" is irrevocably \"off\". \"blocked\" can only be",
+"                 revoked by \"unblock\". (\"device_files\" is dangerous.)",
 "  -extract iso_rr_path disk_path",
 "                 Copy tree under iso_rr_path onto disk address disk_path.",
 "                 This avoids the pitfalls of cp -r addressing rules.",
@@ -1902,7 +2036,7 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "  -check_media_defaults [options] --",
 "                 Preset options for runs of -check_media and -extract_cut.",
 "",
-"Compatibility emulation (argument list may be ended by list delimiter --):",
+"Compatibility emulation (option list may be ended by list delimiter --):",
 "  -as mkisofs  [-help|-version|-o|-R|-r|-J|-V|-P|-f|-m|-exclude-list|",
 "                -no-pad|-M|-C|-graft-points|-path-list|pathspecs|-z|",
 "                -no-emul-boot|-b|-c|-boot-info-table|-boot-load-size|-G|...]",
@@ -1918,7 +2052,7 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "  -pacifier \"xorriso\"|\"cdrecord\"|\"mkisofs\"",
 "              Choose format of UPDATE pacifier during write operations.",
 "",
-"General options:",
+"General commands:",
 "  -help       Print this text",
 "  -abort_on severity   Set the threshhold for events to abort the program.",
 "              Useful severities: NEVER, ABORT, FATAL, FAILURE, SORRY, WARNING",
@@ -1933,18 +2067,22 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "              Behavior \"best_effort\" is most endurant but may produce",
 "              results which are correct only on the first glimpse.",
 "  -dialog \"on\"|\"off\"|\"single_line\"",
-"              After all arguments are processed, enter dialog mode.",
+"              After all program arguments are processed, enter dialog mode.",
 "              \"single_line\" does not support newline characters within",
 "              open quotation marks and no line continuation by trailing \\.",
 "  -page len width  Prompt user after len output lines (0=no prompt).",
 "              width (default 80) can adjust line number computation",
 "              to the output terminal's line width.",
 #ifdef Xorriso_with_readlinE
-"  -use_stdin  Use raw standard input even if libreadline is available",
-"  -use_readline  Use libreadline for dialog if available",
+"  -use_readline \"on\"|\"off\"",
+"              Whether to use libreadline for dialog if available.",
 "  -history text  Copy text into libreadline history. This command",
 "              itself is not copied to the history list.",
 #endif /* Xorriso_with_readlinE */
+"  -sh_style_result \"on\"|\"off\"",
+"              If \"on\" do not wrap file addresses in quotation marks with",
+"                -pwd -pwdx -ls -lsd -lsl -lsdl -lsx -lsdx -lslx -lsdlx",
+"                -du -dus -dux -dusx -findx -find",
 "  -backslash_codes \"on\"|\"off\"|",
 "                   \"in_double_quotes\"|\"in_quotes\"|\"with_quoted_input\"",
 "                   [:\"with_program_arguments\"][:\"encode_output\"]",
@@ -1959,19 +2097,35 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "              packet payload. Example of a info message with newline:",
 "                I:1: enter option text :",
 "              -pkt_output:on is intended for use by frontend programs.",
+"  -msg_op \"start_sieve\"|\"read_sieve\"|\"clear_sieve\"|\"end_sieve\"|",
+"          \"parse\"|\"parse_silently\"|\"parse_bulk\"|\"parse_bulk_silently\"|",
+"          \"compare_sev\"|\"list_sev\" param_text",
+"              Enable, use, or disable message sieve. Or parse lines into",
+"              words. Or compare or list severity names.",
+"  -named_pipes_loop mode[:mode]",
+"                    disk_path_stdin disk_path_stdout disk_path_stderr",
+"              Enter an EOF resistant dialog loop at a named pipe as input",
+"              and zero, one or two named pipes as output. \"-\" = no pipe.",
+"              Mode \"cleanup\" removes pipes at loop end. \"keep\" does not.",
+"              Mode \"buffered\" reads all lines from input pipe before it",
+"              opens the output pipes. \"direct\" opens after first input.",
+"  -launch_frontend program [args ...] --",
+"              Start a program, connect its stdin to xorriso stdout and",
+"              stderr, connect its stdout to xorriso stdin.",
+"              Use any given parameters as arguments for the started program.",
 "  -logfile channel fileaddress  Copy output of a channel to the given file.",
 "              channel may be 'R','I','M' as with -pkt_output or '.'",
 "              for the consolidated -pkt_output stream.",
-"  -mark text  If text is not empty it will get put out each time an",
-"              option is completed.",
+"  -mark text  If text is not empty it will get put out each time a command",
+"              is completed.",
 "  -temp_mem_limit number[\"k\"|\"m\"]",
 "              Set the maximum size for pattern expansion. (Default is 16m)",
 "  -prog text  Use text as this program's name in subsequent messages",
 "  -prog_help text  Use text as this program's name and perform -help",
-"  -status mode|filter  Report the current settings of persistent options.",
+"  -status mode|filter  Report the current settings of persistent commands.",
 "              Modes:",
-"                 short... print only important or altered options",
-"                 long ... print options even if they have default settings",
+"                 short... print only important or altered settings",
+"                 long ... print settings even if they have default values",
 "                 long_history  like long plus -history: lines",
 "              Filters begin with '-' and are compared literally against the",
 "              output lines of -status long_history. A line is put out only",
@@ -1979,10 +2133,11 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "  -status_history_max number  Maximum number of history lines to be reported",
 "              with -status:long_history",
 "  -options_from_file fileaddress",
-"              Reads lines from the given file and executes them as program",
-"              options.",
-"  -no_rc      Only if used as first command line argument this option",
-"              prevents reading and interpretation of startup files.",
+"              Reads lines from the given file and executes them as commands.",
+"  -no_rc      Only if used as first program argument, this command",
+"              prevents reading and interpretation of these startup files:",
+"               /etc/default/xorriso , /etc/opt/xorriso/rc",
+"               /etc/xorriso/xorriso.conf , $HOME/.xorrisorc",
 "  -print text",
 "              Print a text to result channel.",
 "  -print_info text",
@@ -2006,19 +2161,20 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "              Tell whether certain extra features were enabled at compile",
 "              time. Code \"all\" lists all features and a headline. Other",
 "              codes pick a single feature. \"codes\" lists the known codes.",
+"  -list_arg_sorting",
+"              Print the sorting order of xorriso commands with option -x.",
 "  -version    Tell program and version number",
 "  -end        End program. Commit eventual pending changes.",
 "  -rollback_end",
 "              End program. Discard pending changes.",
 "",
 "",
-"Option -page causes a user prompt after the given number of result lines.",
+"Command -page causes a user prompt after the given number of result lines.",
 "Empty input resumes output until the next prompt. Other input may be:",
 "  @     suppresses paging until the current action is done",
 "  @@    suppresses further result output but continues the action",
 "  @@@   aborts the current action",
-"  other aborts the current action and executes input as new",
-"        option",
+"  other aborts the current action and executes input as new command",
 "",
 
 #endif /* ! Xorriso_no_helP */
@@ -2052,6 +2208,22 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 }
 
 
+/* Option -hfsplus "on"|"off" */
+int Xorriso_option_hfsplus(struct XorrisO *xorriso, char *mode, int flag)
+{
+ if(strcmp(mode, "off")==0)
+   xorriso->do_hfsplus= 0;
+ else if(strcmp(mode, "on")==0)
+   xorriso->do_hfsplus= 1;
+ else {
+   sprintf(xorriso->info_text, "-hfsplus: unknown mode '%s'", mode);
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+   return(0);
+ }
+ return(1);
+}
+
+
 /* Option -hide */
 int Xorriso_option_hide(struct XorrisO *xorriso, char *hide_state,
                         int argc, char **argv, int *idx, int flag)
@@ -2095,7 +2267,7 @@ ex:;
 /* Option -history */
 int Xorriso_option_history(struct XorrisO *xorriso, char *line, int flag)
 {
- Xorriso_dialog_input(xorriso,line,strlen(line)+1,2);
+ Xorriso_dialog_input(xorriso, line, strlen(line) + 1, 2 | 32);
  return(1);
 }
 
