@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2012 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2013 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -33,6 +33,10 @@
 /* Option -pacifier */
 int Xorriso_option_pacifier(struct XorrisO *xorriso, char *style, int flag)
 {
+
+#define Xorriso_pacifier_min_intvL 0.1
+#define Xorriso_pacifier_max_intvL 60.0
+
  if(strcmp(style, "xorriso")==0 || strcmp(style, "default")==0)
    xorriso->pacifier_style= 0;
  else if(strcmp(style, "mkisofs")==0 || strcmp(style, "genisofs")==0 ||
@@ -41,7 +45,24 @@ int Xorriso_option_pacifier(struct XorrisO *xorriso, char *style, int flag)
  else if(strcmp(style, "cdrecord")==0 || strcmp(style, "cdrskin")==0 ||
          strcmp(style, "wodim")==0 || strcmp(style, "xorrecord")==0)
    xorriso->pacifier_style= 2;
- else {
+
+ else if(strncmp(style, "interval=", 9) == 0) {
+   sscanf(style + 9, "%lf", &(xorriso->pacifier_interval));
+   if(xorriso->pacifier_interval < Xorriso_pacifier_min_intvL) {
+     sprintf(xorriso->info_text,
+             "-pacifier: interval='%s' is too low. Min: %f",
+             style, Xorriso_pacifier_min_intvL);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "WARNING", 0);
+     xorriso->pacifier_interval= Xorriso_pacifier_min_intvL;
+   } else if(xorriso->pacifier_interval > Xorriso_pacifier_max_intvL) {
+     sprintf(xorriso->info_text,
+             "-pacifier: interval='%s' is too high. Max: %f",
+             style, Xorriso_pacifier_max_intvL);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "WARNING", 0);
+     xorriso->pacifier_interval= Xorriso_pacifier_max_intvL;
+   }
+
+ } else {
    sprintf(xorriso->info_text, "-pacifier: unknown behavior code '%s'", style);
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "SORRY", 0);
    return(0);
@@ -395,8 +416,10 @@ int Xorriso_option_pwdi(struct XorrisO *xorriso, int flag)
 {
  sprintf(xorriso->info_text,"current working directory in ISO image:\n");
  Xorriso_info(xorriso,0);
- Text_shellsafe(xorriso->wdi, xorriso->result_line, 0);
- strcat(xorriso->result_line, "/\n");
+ Xorriso_esc_filepath(xorriso, xorriso->wdi, xorriso->result_line, 0);
+ if(xorriso->sh_style_result == 0 || xorriso->wdi[0] == 0)
+   strcat(xorriso->result_line, "/");
+ strcat(xorriso->result_line, "\n");
  Xorriso_result(xorriso,0);
  return(1);
 }
@@ -407,7 +430,10 @@ int Xorriso_option_pwdx(struct XorrisO *xorriso, int flag)
 {
  sprintf(xorriso->info_text,"current working directory on hard disk:\n");
  Xorriso_info(xorriso,0);
- sprintf(xorriso->result_line,"%s/\n",xorriso->wdx);
+ Xorriso_esc_filepath(xorriso, xorriso->wdx, xorriso->result_line, 0);
+ if(xorriso->sh_style_result == 0 || xorriso->wdx[0] == 0)
+   strcat(xorriso->result_line, "/");
+ strcat(xorriso->result_line, "\n");
  Xorriso_result(xorriso,0);
  return(1);
 }
@@ -603,6 +629,7 @@ int Xorriso_option_rollback(struct XorrisO *xorriso, int flag)
 
  if(Sfile_str(indev, xorriso->indev, 0)<=0)
    {ret= -1; goto ex;}
+ xorriso->volset_change_pending= 0;
  ret= Xorriso_give_up_drive(xorriso, 1|8);
  if(ret<=0)
    goto ex;
@@ -620,7 +647,6 @@ int Xorriso_option_rom_toc_scan(struct XorrisO *xorriso, char *mode, int flag)
  int l;
  char *cpt, *npt;
 
- xorriso->toc_emulation_flag= 0;
  npt= cpt= mode;
  for(cpt= mode; npt != NULL; cpt= npt + 1) {
    npt= strchr(cpt,':');
@@ -629,20 +655,22 @@ int Xorriso_option_rom_toc_scan(struct XorrisO *xorriso, char *mode, int flag)
    else
      l= npt-cpt;
    if(l==0)
-     goto unknown_mode;
-   xorriso->toc_emulation_flag&= ~4;
+ continue;
    if(strncmp(cpt, "off", l) == 0)
-     xorriso->toc_emulation_flag&= ~1;
+     xorriso->toc_emulation_flag&= ~5;
    else if(strncmp(cpt, "on", l) == 0)
-     xorriso->toc_emulation_flag|= 1;
+     xorriso->toc_emulation_flag= (xorriso->toc_emulation_flag & ~4) | 1;
    else if(strncmp(cpt, "force", l) == 0)
-     xorriso->toc_emulation_flag|= 4;
+     xorriso->toc_emulation_flag|= 5;
    else if(strncmp(cpt, "emul_off", l) == 0)
      xorriso->toc_emulation_flag|= 2;
    else if(strncmp(cpt, "emul_on", l) == 0)
      xorriso->toc_emulation_flag&= ~2;
+   else if(strncmp(cpt, "emul_wide", l) == 0)
+     xorriso->toc_emulation_flag|= 8;
+   else if(strncmp(cpt, "emul_narrow", l) == 0)
+     xorriso->toc_emulation_flag&= ~8;
    else {
-unknown_mode:;
      sprintf(xorriso->info_text, "-rom_toc_scan: unknown mode in '%s'", mode);
      Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
      return(0);
@@ -1200,6 +1228,23 @@ ex:;
 }
 
 
+/* Option -sh_style_result */
+int Xorriso_option_sh_style_result(struct XorrisO *xorriso, char *mode,
+                                   int flag)
+{
+ if(strcmp(mode, "off") == 0) {
+   xorriso->sh_style_result= 0;
+ } else if(strcmp(mode, "on") == 0) {
+   xorriso->sh_style_result= 1;
+ } else {
+   sprintf(xorriso->info_text, "-sh_style_result: unknown mode '%s'", mode);
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+   return(0);
+ }
+ return(1); 
+}
+
+
 /* Option -signal_handling */
 /* @param flag bit0= prescan mode: do not yet install the eventual handler
                      else: when switching from other modes to "off":
@@ -1485,6 +1530,8 @@ int Xorriso_option_temp_mem_limit(struct XorrisO *xorriso, char *size,
 
 
 /* Option -toc */
+/* @param flag   bit0= short report form as with -dev, no table-of-content
+*/
 int Xorriso_option_toc(struct XorrisO *xorriso, int flag)
 {
  int ret, in_ret= 1000;
@@ -1500,7 +1547,7 @@ int Xorriso_option_toc(struct XorrisO *xorriso, int flag)
    }
    ret= 1;
    if(xorriso->outdev[0]!=0)
-     ret= Xorriso_toc(xorriso, 2);
+     ret= Xorriso_toc(xorriso, 2 | (flag & 1));
    if(in_ret<ret)
      ret= in_ret;
  }
@@ -1508,6 +1555,44 @@ int Xorriso_option_toc(struct XorrisO *xorriso, int flag)
 }
 
 
+/* Option -toc_of */
+int Xorriso_option_toc_of(struct XorrisO *xorriso, char *which, int flag)
+{
+ int ret= 0, toc_flag= 0;
+
+ if(strstr(which, ":short") != NULL)
+   toc_flag|= 1;
+ if(strncmp(which, "in", 2) == 0) {
+   if(xorriso->indev[0] == 0) {
+     Xorriso_msgs_submit(xorriso, 0, "-toc_of 'in' : No input drive aquired",
+                         0, "NOTE", 0);
+     return(2);
+   }
+   ret= Xorriso_toc(xorriso, toc_flag | 0);
+ } else if(strncmp(which, "out", 3) == 0) {
+   if(xorriso->outdev[0] == 0) {
+     Xorriso_msgs_submit(xorriso, 0, "-toc_of 'out' : No output drive aquired",
+                         0, "NOTE", 0);
+     return(2);
+   }
+   ret= Xorriso_toc(xorriso, toc_flag | 2);
+ } else if(strncmp(which, "all", 3) == 0) {
+   if(xorriso->indev[0] == 0 && xorriso->outdev[0] == 0) {
+     Xorriso_msgs_submit(xorriso, 0, "-toc_of 'all' : No drive aquired",
+                         0, "NOTE", 0);
+     return(2);
+   }
+   ret= Xorriso_option_toc(xorriso, toc_flag | 0);
+ } else {
+   sprintf(xorriso->info_text, "-toc_of: Unknown drive code ");
+   Text_shellsafe(which, xorriso->info_text, 1);
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "SORRY", 0);
+   ret= 0;
+ }
+ return(ret);
+}
+
+
 /* Option -uid */
 int Xorriso_option_uid(struct XorrisO *xorriso, char *uid, int flag)
 {
@@ -1774,15 +1859,16 @@ int Xorriso_option_version(struct XorrisO *xorriso, int flag)
  sprintf(xorriso->result_line,
 "ISO 9660 Rock Ridge filesystem manipulator and CD/DVD/BD burn program\n");
  sprintf(xorriso->result_line+strlen(xorriso->result_line),
-"Copyright (C) 2012, Thomas Schmitt <scdbackup@gmx.net>, libburnia project.\n");
+"Copyright (C) 2013, Thomas Schmitt <scdbackup@gmx.net>, libburnia project.\n");
  Xorriso_result(xorriso, 0);
  sprintf(xorriso->result_line,
          "xorriso version   :  %d.%d.%d%s\n",
          Xorriso_header_version_majoR, Xorriso_header_version_minoR,
          Xorriso_header_version_micrO, Xorriso_program_patch_leveL);
- sprintf(xorriso->result_line+strlen(xorriso->result_line),
-         "Version timestamp :  %s\n",Xorriso_timestamP);
- sprintf(xorriso->result_line+strlen(xorriso->result_line),
+ Xorriso_result(xorriso, 0);
+ sprintf(xorriso->result_line, "Version timestamp :  %s\n",Xorriso_timestamP);
+ Xorriso_result(xorriso, 0);
+ sprintf(xorriso->result_line,
          "Build timestamp   :  %s\n",Xorriso_build_timestamP);
  Xorriso_result(xorriso, 0);
  Xorriso_report_lib_versions(xorriso, 0);
