@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2011 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2013 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -30,12 +30,16 @@
 #include <sys/wait.h>
 
 
-
 #include "xorriso.h"
 #include "xorriso_private.h"
 #include "xorrisoburn.h"
 
 
+#ifdef Xorriso_fetch_with_msg_queueS
+#include <pthread.h>
+#endif
+
+
 /* @param flag bit0= do not warn of wildcards
                bit1= these are disk_paths
 */
@@ -207,17 +211,55 @@ int Xorriso_set_problem_status(struct XorrisO *xorriso, char *severity,
  char *sev_text= "ALL";
  int sev, ret;
 
+#ifdef Xorriso_fetch_with_msg_queueS
+ int locked= 0, uret;
+ static int complaints= 0, complaint_limit= 5;
+#endif
+
  if(severity[0])
    sev_text= severity;
  ret= Xorriso__text_to_sev(sev_text, &sev, 0);
  if(ret<=0)
    return(0);
+
+#ifdef Xorriso_fetch_with_msg_queueS
+
+ ret= pthread_mutex_lock(&(xorriso->problem_status_lock));
+ if(ret != 0) {
+   /* Cannot report failure through the failing message output system */
+   complaints++;
+   if(complaints < complaint_limit)
+     fprintf(stderr,
+        "xorriso : pthread_mutex_lock() for problem_status returns %d\n",
+        ret);
+ } else
+   locked= 1;
+
+#endif /* Xorriso_fetch_with_msg_queueS */
+
  xorriso->problem_status= sev;
  strcpy(xorriso->problem_status_text, sev_text);
  if(sev > xorriso->eternal_problem_status) {
    xorriso->eternal_problem_status= sev;
    strcpy(xorriso->eternal_problem_status_text, sev_text);
  }
+
+#ifdef Xorriso_fetch_with_msg_queueS
+
+ if(locked) {
+   uret= pthread_mutex_unlock(&(xorriso->problem_status_lock));
+   if(uret != 0) {
+     /* Cannot report failure through the failing message output system */
+     complaints++;
+     if(complaints < complaint_limit)
+       fprintf(stderr,
+            "xorriso : pthread_mutex_unlock() for problem_status returns %d\n",
+            uret);
+   }
+ }
+
+#endif /* Xorriso_fetch_with_msg_queueS */
+
  return(1);
 }
 
@@ -292,12 +334,12 @@ int Xorriso_cpmv_args(struct XorrisO *xorriso, char *cmd,
  }
 
  ret= Xorriso_opt_args(xorriso, cmd, argc, argv, *idx, &end_idx, optc, optv,
-                       (flag&2)|16); /* ignore last argument */
+                       1 | (flag&2) | 16); /* ignore last argument */
  if(ret<=0)
    goto ex;
  /* demand one match, or 0 with a constant */
  ret= Xorriso_opt_args(xorriso, cmd, argc, argv, end_idx, &end_idx, &destc,
-                       &destv, ((flag&4)>>1) | 32 | 64);
+                       &destv, 1 | ((flag&4)>>1) | 32 | 64);
  if(ret<=0)
    goto ex;
 
@@ -450,41 +492,44 @@ int Xorriso_count_args(struct XorrisO *xorriso, int argc, char **argv,
     "ban_stdio_write","close_filter_list","commit",
     "device_links","devices","end",
     "for_backup", "help",
-    "list_formats","list_speeds",
+    "list_arg_sorting","list_formats","list_speeds",
     "no_rc","print_size","pvd_info","pwd","pwdi","pwdx",
     "read_mkisofsrc","rollback","rollback_end",
     "tell_media_space","toc","version",
     ""
  };
  static char arg1_commands[][40]= {
-    "abort_on","acl","add_plainly","application_id","auto_charset",
-    "abstract_file",
+    "abort_on","acl","add_plainly","application_id","application_use",
+    "auto_charset","abstract_file",
     "backslash_codes","blank","biblio_file",
-    "calm_drive","cd","cdi","cdx","charset","close","close_damaged",
+    "calm_drive","cd","cdi","cdx","changes_pending","charset",
+    "close","close_damaged",
     "commit_eject","compliance","copyright_file",
     "dev","dialog","disk_dev_ino","disk_pattern","displacement",
     "dummy","dvd_obs","early_stdio_test", "eject",
     "iso_rr_pattern","follow","format","fs","gid","grow_blindly","hardlinks",
-    "history","indev","in_charset","joliet",
+    "hfsplus","history","indev","in_charset","joliet",
     "list_delimiter","list_extras","list_profiles","local_charset",
     "mark","md5","mount_opts","not_leaf","not_list","not_mgt",
     "options_from_file","osirrox","outdev","out_charset","overwrite",
     "pacifier","padding","path_list","pathspecs","pkt_output",
     "preparer_id","print","print_info","print_mark","prompt",
     "prog","prog_help","publisher","quoted_not_list","quoted_path_list",
-    "reassure","report_about","rom_toc_scan","scsi_log",
-    "session_log","signal_handling","sleep",
+    "reassure","report_about","rockridge",
+    "rom_toc_scan","rr_reloc_dir","scsi_log",
+    "session_log","sh_style_result","signal_handling","sleep",
     "speed","split_size","status","status_history_max",
-    "stdio_sync","stream_recording","system_id","temp_mem_limit",
+    "stdio_sync","stream_recording","system_id","temp_mem_limit","toc_of",
     "uid","unregister_filter","use_readline","volid","volset_id",
-    "xattr","zisofs",
+    "write_type","xattr","zisofs",
     ""
  };
  static char arg2_commands[][40]= {
     "assert_volid","boot_image","clone","compare","compare_r","drive_class",
+    "data_cache_size",
     "errfile_log","error_behavior","extract","extract_single",
-    "jigdo","load","logfile",
-    "map","map_single","page","return_with",
+    "jigdo","lns","lnsi","load","logfile",
+    "map","map_single","move","msg_op","page","return_with",
     "scdbackup_tag","update","update_r","volume_date",
     ""
  };
@@ -493,7 +538,8 @@ int Xorriso_count_args(struct XorrisO *xorriso, int argc, char **argv,
     ""
  };
  static char arg4_commands[][40]= {
-    "cut_out","extract_cut","mount","mount_cmd","paste_in","session_string",
+    "cut_out","extract_cut","mount","mount_cmd","named_pipe_loop",
+    "paste_in","session_string",
     ""
  };
  static char argn_commands[][40]= {
@@ -506,7 +552,7 @@ int Xorriso_count_args(struct XorrisO *xorriso, int argc, char **argv,
     "file_size_limit","find","findi","finds","findx",
     "getfacl","getfacli","getfacl_r","getfacl_ri",
     "getfattr","getfattri","getfattr_r","getfattr_ri","hide",
-    "ls","lsi","lsl","lsli","lsd","lsdi","lsdl","lsdli",
+    "launch_frontend","ls","lsi","lsl","lsli","lsd","lsdi","lsdl","lsdli",
     "lsx","lslx","lsdx","lsdlx","map_l","mv","mvi","mkdir","mkdiri",
     "not_paths","rm","rmi","rm_r","rm_ri","rmdir","rmdiri","update_l",
     "setfacl","setfacli","setfacl_list","setfacl_listi",
@@ -571,9 +617,283 @@ ex:
 }
 
 
+/* @param flag bit0= list sorting order rather than looking for argv[idx]
+*/
+int Xorriso_cmd_sorting_rank(struct XorrisO *xorriso,
+                        int argc, char **argv, int idx, int flag)
+{
+ int ret, i, cmd_data_size= 2 * SfileadrL;
+ char *cmd, *cmd_data= NULL;
+ static char *commands[]= {
+
+   "* Execution order of program arguments with option -x:",
+   "x",
+
+   "* Support for frontend programs via stdin and stdout (1):",
+   "prog", "prog_help",
+
+   "* Exception processing:",
+   "abort_on", "return_with", "report_about", "signal_handling",
+   "error_behavior",
+
+   "* Scripting, dialog and program control features (1):",
+   "no_rc", "help", "version", "list_extras", "list_arg_sorting",
+   "temp_mem_limit", "backslash_codes",
+   "errfile_log", "session_log", "scsi_log",
+   "options_from_file", "list_delimiter",
+   "print", "print_info", "print_mark", "prompt", "sleep",
+   "sh_style_result",
+
+   "* Drive and media related inquiry actions (1):",
+   "devices", "device_links",
+   "mount_opts", "mount_cmd", "session_string",
+
+   "* Influencing the behavior of image loading:",
+   "load", "displacement", "drive_class", "assert_volid", "in_charset",
+   "auto_charset", "hardlinks", "acl", "xattr", "md5", "for_backup",
+   "disk_dev_ino", "rom_toc_scan", "calm_drive", "ban_stdio_write",
+   "early_stdio_test", "data_cache_size",
+
+   "* Character sets:",
+   "charset", "local_charset",
+
+   "* Aquiring source and target drive:",
+   "dev", "indev", "outdev",
+
+   "* Drive and media related inquiry actions (2):",
+   "list_profiles", "list_formats", "list_speeds",
+   "toc", "toc_of", "pvd_info",
+
+   "* Settings for file insertion:",
+   "file_size_limit", "not_mgt", "not_paths", "not_leaf", "not_list",
+   "quoted_not_list", "follow", "pathspecs", "overwrite", "split_size",
+
+   "* Navigation in ISO image and disk filesystem (1):",
+   "cd", "cdx", "pwd", "pwdx",
+
+   "* Inserting files into ISO image:",
+   "disk_pattern", "add_plainly",
+   "mkdir", "lns", "add", "path_list", "quoted_path_list",
+   "map", "map_single", "map_l", "update", "update_r", "update_l",
+   "cut_out", "cpr", 
+   "clone", "cp_clone",
+
+   "* Navigation in ISO image and disk filesystem (2):",
+   "ls", "lsd", "lsl", "lsdl", "lsx", "lsdx", "lslx", "lsdlx",
+   "getfacl", "getfacl_r", "getfattr", "getfattr_r", "du", "dus",
+   "dux", "dusx", "findx",
+   "compare", "compare_r", "compare_l", "show_stream", "show_stream_r",
+
+   "* File manipulations:",
+   "iso_rr_pattern",
+   "rm", "rm_r", "rmdir", "move", "mv",
+   "chown", "chown_r", "chgrp", "chgrp_r", "chmod", "chmod_r", "setfacl",
+   "setfacl_r", "setfacl_list", "setfattr", "setfattr_r", "setfattr_list",
+   "alter_date", "alter_date_r", "hide",
+
+   "* Filters for data file content:",
+   "external_filter", "unregister_filter", "close_filter_list",
+   "set_filter", "set_filter_r", 
+
+   "* Tree traversal command -find:",
+   "find",
+
+   "* osirrox ISO-to-disk restore options:",
+   "osirrox", "extract", "extract_single", "extract_l", "extract_cut",
+   "cpx", "cpax", "cp_rx", "cp_rax", "paste_in",
+   "mount",
+
+   "* Settings for result writing:",
+   "rockridge", "joliet", "hfsplus","compliance", "rr_reloc_dir",
+   "volid", "volset_id", "publisher",
+   "application_id", "system_id", "volume_date", "copyright_file",
+   "abstract_file", "biblio_file", "preparer_id", "application_use",
+   "out_charset", "read_mkisofsrc",
+   "uid", "gid", "zisofs", "speed", "stream_recording", "dvd_obs",
+   "stdio_sync", "dummy", "fs", "close", "padding", "write_type",
+   "grow_blindly", "pacifier", "scdbackup_tag",
+
+   "* Bootable ISO images:",
+   "boot_image", "append_partition",
+
+   "* Jigdo Template Extraction:",
+   "jigdo",
+
+   "* Command compatibility emulations:",
+   "as",
+
+   "* Scripting, dialog and program control features (2):",
+   "history", "status_history_max", "status",
+
+   "* Drive and media related inquiry actions (3):",
+   "print_size", "tell_media_space",
+
+   "* Writing the result, drive control:",
+   "format", "blank", "close_damaged",
+   "rollback", "changes_pending", "commit", "commit_eject",
+   "eject",
+
+   "* Evaluation of readability and recovery:",
+   "check_media_defaults", "check_media", "check_md5", "check_md5_r",
+
+   "* Support for frontend programs via stdin and stdout (2):",
+   "pkt_output", "logfile", "mark", "msg_op",
+
+   "* Dialog mode control:",
+   "dialog", "page", "use_readline", "reassure",
+
+   "* Support for frontend programs via stdin and stdout (3):",
+   "launch_frontend", "named_pipe_loop",
+
+   "* Scripting, dialog and program control features (3):",
+   "rollback_end", "end",
+
+   ""
+ };
+
+ if(flag & 1) {
+   for(i= 0; commands[i][0] !=0; i++) {
+     if(commands[i][0] == '*')
+       sprintf(xorriso->result_line, "#%s\n", commands[i] + 1);
+     else
+       sprintf(xorriso->result_line, "-%s\n", commands[i]);
+     Xorriso_result(xorriso, 0);
+   }
+   ret= 1; goto ex;
+ }
+ if(argc <= 0)
+   {ret= -1; goto ex;}
+
+ Xorriso_alloc_meM(cmd_data, char, cmd_data_size);
+ ret= Xorriso_normalize_command(xorriso, argv[idx], -1,
+                                cmd_data, cmd_data_size, &cmd, 0);
+ if(ret < 0)
+   goto ex;
+
+ if(cmd[0] == '#' || cmd[0] == 0 ||
+    strcmp(cmd, xorriso->list_delimiter) == 0) {
+   /* Move to end: comment line , empty option , orphaned list delimiter */
+   ret= 0x7fffffff; goto ex;
+ }
+ for(i= 0; commands[i][0] !=0; i++) {
+   if(commands[i][0] == '*') /* headline in command list */
+ continue;
+   if(strcmp(commands[i], cmd) != 0)
+ continue;
+   ret= i + 1; goto ex;
+ }
+
+ ret= 1;
+ex:
+ Xorriso_free_meM(cmd_data);
+ return(ret);
+}
+
+
+int Xorriso__cmp_cmd_rank(const void *a, const void *b)
+{
+ int ra, rb;
+
+ ra= ((int *) a)[1];
+ rb= ((int *) b)[1];
+ if(ra < rb)
+   return(-1);
+ if(ra > rb)
+   return(1);
+ ra= ((int *) a)[2];
+ rb= ((int *) b)[2];
+ if(ra < rb)
+   return(-1);
+ if(ra > rb)
+   return(1);
+ return(0);
+}
+
+
+/* @param flag bit0= print command sequence rather than executing it
+               bit1= these are command line arguments
+                     (for xorriso->argument_emulation)
+*/
+int Xorriso_exec_args_sorted(struct XorrisO *xorriso,
+                             int argc, char **argv, int *idx, int flag)
+{
+ int cmd_count= 0, ret, i, arg_count, *idx_rank= NULL, cmd_idx;
+
+ /* Count commands and allocate index-rank array */
+ for(i= *idx; i < argc; i++) {
+   ret= Xorriso_count_args(xorriso, argc - i, argv + i, &arg_count, 1);
+   if(ret <= 0)
+     goto ex;
+   if(ret != 1)
+ continue;
+   cmd_count++;
+   i+= arg_count;
+ }
+ if(cmd_count <= 0)
+   {ret= 1; goto ex;}
+ Xorriso_alloc_meM(idx_rank, int, 3 * cmd_count);
+
+ /* Fill index-rank array and sort */
+ cmd_count= 0;
+ for(i= *idx; i < argc; i++) {
+   ret= Xorriso_count_args(xorriso, argc - i, argv + i, &arg_count, 1);
+   if(ret <= 0)
+     goto ex;
+   if(ret != 1)
+ continue;
+   idx_rank[3 * cmd_count]= i;
+   ret= Xorriso_cmd_sorting_rank(xorriso, argc, argv, i, 0);
+   if(ret < 0)
+     goto ex;
+   idx_rank[3 * cmd_count + 1]= ret;
+   idx_rank[3 * cmd_count + 2]= cmd_count;
+   cmd_count++;
+   i+= arg_count;
+ }
+ qsort(idx_rank, cmd_count, 3 * sizeof(int), Xorriso__cmp_cmd_rank);
+
+ /* Execute or print indice from index-rank array */
+ if(flag & 1) {
+   sprintf(xorriso->result_line,
+           "Automatically determined command sequence:\n");
+   Xorriso_result(xorriso, 0);
+   xorriso->result_line[0]= 0;
+ }
+ for(i= 0; i < cmd_count; i++) {
+   cmd_idx= idx_rank[3 * i];
+   if(flag & 1) {
+     if(strlen(xorriso->result_line) + 1 + strlen(argv[cmd_idx]) > 78) {
+       strcat(xorriso->result_line, "\n");
+       Xorriso_result(xorriso, 0);
+       xorriso->result_line[0]= 0;
+     }
+     sprintf(xorriso->result_line + strlen(xorriso->result_line),
+             " %s", argv[cmd_idx]);
+   } else {
+     ret= Xorriso_interpreter(xorriso, argc, argv, &cmd_idx, 4 | (flag & 2));
+     if(ret <= 0 || ret == 3)
+       goto ex;
+   }
+ }
+ if(flag & 1) {
+   if(strlen(xorriso->result_line) > 0) {
+     strcat(xorriso->result_line, "\n");
+     Xorriso_result(xorriso, 0);
+   }
+ } else
+   *idx= argc;
+ ret= 1;
+ex:
+ Xorriso_free_meM(idx_rank); 
+ return(ret);
+}
+
+
 /* @param flag bit0= recursion
                bit1= these are command line arguments
                      (for xorriso->argument_emulation)
+               bit2= Only execute the one command argv[*idx] and advance 
+                     *idx to the next command if sucessful. Then return.
 */
 int Xorriso_interpreter(struct XorrisO *xorriso,
                         int argc, char **argv, int *idx, int flag)
@@ -584,7 +904,7 @@ return:
 {
  int ret, was_dashed, end_ret, num1, num2, cmd_data_size= 2 * SfileadrL;
  int mem_idx, arg_count, i;
- char *cmd, *original_cmd, *cmd_data= NULL, *arg1, *arg2;
+ char *cmd, *original_cmd, *cmd_data= NULL, *arg1, *arg2, *arg3, *arg4;
  
  Xorriso_alloc_meM(cmd_data, char, cmd_data_size);
 
@@ -610,15 +930,19 @@ next_command:;
      if((*idx)>=argc)
        {ret= 1; goto ex;}
    }
+   if((xorriso->arrange_args || (flag & 8)) && !(flag & (4 | 16))) {
+     ret= Xorriso_exec_args_sorted(xorriso, argc, argv, idx, 0);
+     goto ex;
+   } 
  }
 
  ret= Xorriso_count_args(xorriso, argc - *idx, argv + *idx, &arg_count, 1);
  if((ret == 1 || ret == 2) &&
     strcmp(argv[*idx], xorriso->list_delimiter) != 0) {
-   sprintf(xorriso->info_text, "Command:  %s", argv[*idx]);
+   sprintf(xorriso->info_text, "Command:    %s", argv[*idx]);
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "DEBUG", 0);
    for(i= 1; i <= arg_count && *idx + i < argc; i++) {
-     sprintf(xorriso->info_text, "Argument:    %s", argv[*idx + i]);
+     sprintf(xorriso->info_text, "Parameter:     %s", argv[*idx + i]);
      Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "DEBUG", 0);
    }
    if(*idx + arg_count >= argc) {
@@ -655,6 +979,10 @@ next_command:;
    arg2= argv[(*idx)+1];
  else
    arg2= "";
+ if((*idx) + 2 < argc)
+   arg3= argv[(*idx) + 2];
+ else
+   arg3= "";
 
  ret= 1;
  if(cmd[0]=='#' || cmd[0]==0) {
@@ -686,13 +1014,16 @@ next_command:;
 
  } else if(strcmp(cmd,"append_partition")==0) {
    (*idx)+= 3;
-   ret= Xorriso_option_append_partition(xorriso, arg1, arg2, argv[*idx - 1],
-                                        0);
+   ret= Xorriso_option_append_partition(xorriso, arg1, arg2, arg3, 0);
 
  } else if(strcmp(cmd,"application_id")==0) {
    (*idx)++;
    ret= Xorriso_option_application_id(xorriso, arg1, 0);
 
+ } else if(strcmp(cmd,"application_use") == 0) {
+   (*idx)++;
+   ret= Xorriso_option_application_use(xorriso, arg1, 0);
+
  } else if(strcmp(cmd,"as")==0) {
    ret= Xorriso_option_as(xorriso, argc, argv, idx, 0);
 
@@ -735,6 +1066,10 @@ next_command:;
    (*idx)++;
    ret= Xorriso_option_cdx(xorriso, arg1, 0);
 
+ } else if(strcmp(cmd, "changes_pending")==0) {
+   (*idx)++;
+   ret= Xorriso_option_changes_pending(xorriso, arg1, 0);
+
  } else if(strcmp(cmd,"charset")==0) {
    (*idx)++;
    ret= Xorriso_option_charset(xorriso, arg1, 3);
@@ -841,6 +1176,10 @@ next_command:;
      ret= Xorriso_option_cut_out(xorriso, arg1, arg2,
                                  argv[(*idx)-2], argv[(*idx)-1], 0);
 
+ } else if(strcmp(cmd,"data_cache_size")==0) {
+   (*idx)+= 2;
+   ret= Xorriso_option_data_cache_size(xorriso, arg1, arg2, 0);
+
  } else if(strcmp(cmd,"dev")==0) {
    (*idx)++;
    ret= Xorriso_option_dev(xorriso, arg1, 3);
@@ -987,6 +1326,10 @@ next_command:;
    (*idx)++;
    ret= Xorriso_option_hardlinks(xorriso, arg1, 0);
 
+ } else if(strcmp(cmd,"hfsplus")==0) {
+   (*idx)++;
+   ret= Xorriso_option_hfsplus(xorriso, arg1, 0);
+
  } else if(strcmp(cmd,"help")==0) {
    Xorriso_option_help(xorriso,0);
 
@@ -1019,6 +1362,12 @@ next_command:;
    (*idx)++;
    ret= Xorriso_option_joliet(xorriso, arg1, 0);
 
+ } else if(strcmp(cmd, "launch_frontend") == 0) {
+   ret= Xorriso_option_launch_frontend(xorriso, argc, argv, idx, 0);
+
+ } else if(strcmp(cmd, "list_arg_sorting") == 0) {
+   ret= Xorriso_option_list_arg_sorting(xorriso, 0);
+
  } else if(strcmp(cmd, "list_delimiter") == 0) {
    (*idx)++;
    ret= Xorriso_option_list_delimiter(xorriso, arg1, 0);
@@ -1037,6 +1386,10 @@ next_command:;
  } else if(strcmp(cmd,"list_speeds")==0) {
    ret= Xorriso_option_list_speeds(xorriso, 0);
 
+ } else if(strcmp(cmd, "lns") == 0 || strcmp(cmd, "lnsi") == 0) {
+   (*idx)+= 2;
+   ret= Xorriso_option_lnsi(xorriso, arg1, arg2, 0);
+
  } else if(strcmp(cmd,"load")==0) {
    (*idx)+= 2;
    ret= Xorriso_option_load(xorriso, arg1, arg2, 0);
@@ -1099,12 +1452,30 @@ next_command:;
    (*idx)++;
    ret= Xorriso_option_mount_opts(xorriso, arg1, 0);
 
+ } else if(strcmp(cmd, "move")==0) {
+   (*idx)+= 2;
+   ret= Xorriso_option_move(xorriso, arg1, arg2, 0);
+
+ } else if(strcmp(cmd,"msg_op") == 0) {
+   (*idx)+= 2;
+   ret= Xorriso_option_msg_op(xorriso, arg1, arg2, 0);
+
  } else if(strcmp(cmd,"mv")==0 || strcmp(cmd,"mvi")==0) {
    ret= Xorriso_option_mvi(xorriso, argc, argv, idx, 0);
 
  } else if(strcmp(cmd,"mkdir")==0 || strcmp(cmd,"mkdiri")==0) {
    ret= Xorriso_option_mkdiri(xorriso, argc, argv, idx, 0);
 
+ } else if(strcmp(cmd,"named_pipe_loop")==0) {
+   if((*idx) + 3 < argc)
+     arg4= argv[(*idx) + 3];
+   else
+     arg4= "";
+   (*idx)+= 4;
+   ret= Xorriso_option_named_pipe_loop(xorriso, arg1, arg2, arg3, arg4, 0);
+   if(ret == 3)
+     goto ex;
+
  } else if(strcmp(cmd,"no_rc")==0) {
    ret= Xorriso_option_no_rc(xorriso, 0);
 
@@ -1263,6 +1634,10 @@ next_command:;
  } else if(strcmp(cmd,"rmdir")==0 || strcmp(cmd,"rmdiri")==0) {
    ret= Xorriso_option_rmi(xorriso, argc, argv, idx, 2);
 
+ } else if(strcmp(cmd, "rockridge") == 0) {
+   (*idx)++;
+   ret= Xorriso_option_rockridge(xorriso, arg1, 0);
+
  } else if(strcmp(cmd,"rollback")==0) {
    ret= Xorriso_option_rollback(xorriso, 0);
 
@@ -1278,6 +1653,10 @@ next_command:;
    (*idx)++;
    Xorriso_option_rom_toc_scan(xorriso, arg1, 0);
 
+ } else if(strcmp(cmd,"rr_reloc_dir")==0) {
+   (*idx)++;
+   Xorriso_option_rr_reloc_dir(xorriso, arg1, 0);
+
  } else if(strcmp(cmd,"scdbackup_tag")==0) {
    (*idx)+= 2;
    ret= Xorriso_option_scdbackup_tag(xorriso, arg1, arg2, 0);
@@ -1335,6 +1714,10 @@ next_command:;
    ret= Xorriso_option_set_filter(xorriso, "", argc, argv, idx,
                                   (strcmp(cmd,"show_stream_r")==0) | 2 | 4);
 
+ } else if(strcmp(cmd,"sh_style_result")==0) {
+   (*idx)++;
+   ret= Xorriso_option_sh_style_result(xorriso, arg1, 0);
+
  } else if(strcmp(cmd,"signal_handling")==0) {
    (*idx)++;
    ret= Xorriso_option_signal_handling(xorriso, arg1, 0);
@@ -1382,9 +1765,90 @@ next_command:;
  } else if(strcmp(cmd,"test")==0) { /* This option does not exist. */
    /* install temporary test code here */;
 
+if (0) {
+/* Test setup for for Xorriso_push_outlists() et.al. */
+/* Test setup for Xorriso_parse_line */
+   int stack_handle = -1, line_count= 0;
+   struct Xorriso_lsT *result_list, *info_list;
+   int Xorriso_process_msg_lists(struct XorrisO *xorriso,
+                                     struct Xorriso_lsT *result_list,
+                                     struct Xorriso_lsT *info_list,
+                                     int *line_count, int flag);
+
+   (*idx)++;
+   if(strcmp(arg1, "push") == 0) {
+     ret= Xorriso_push_outlists(xorriso, &stack_handle, 3);
+     fprintf(stderr, "xorriso -test: push = %d, handle = %d\n",
+             ret, stack_handle);
+   } else if(strcmp(arg1, "pull") == 0) {
+     ret= Xorriso_pull_outlists(xorriso, -1, &result_list, &info_list, 0);
+     fprintf(stderr, "xorriso -test: pull = %d\n", ret);
+     if(ret > 0) {
+       ret= Xorriso_process_msg_lists(xorriso, result_list, info_list,
+                                      &line_count, 0);
+       fprintf(stderr,
+          "xorriso -test: Xorriso_process_msg_lists() = %d, line_count = %d\n",
+          ret, line_count);
+     }
+   } else if(strcmp(arg1, "fetch") == 0) {
+     ret= Xorriso_fetch_outlists(xorriso, -1, &result_list, &info_list, 0);
+     fprintf(stderr, "xorriso -test: fetch = %d\n", ret);
+     if(ret > 0) {
+       ret= Xorriso_process_msg_lists(xorriso, result_list, info_list,
+                                      &line_count, 0);
+       fprintf(stderr,
+          "xorriso -test: Xorriso_process_msg_lists() = %d, line_count = %d\n",
+          ret, line_count);
+     }
+   } else if(strcmp(arg1, "peek") == 0) {
+     ret= Xorriso_peek_outlists(xorriso, -1, 0, 0);
+     fprintf(stderr, "xorriso -test: peek = %d\n", ret);
+   } else if(strcmp(arg1, "sleep_peek") == 0) {
+     usleep(1000000);
+     ret= Xorriso_peek_outlists(xorriso, -1, 0, 0);
+     fprintf(stderr, "xorriso -test: sleep_peek = %d\n", ret);
+   } else if(strcmp(arg1, "peek_loop") == 0) {
+     ret= Xorriso_peek_outlists(xorriso, -1, 3, 4);
+     fprintf(stderr, "xorriso -test: peek_loop = %d\n", ret);
+   } else if(strcmp(arg1, "start") == 0) {
+     ret= Xorriso_start_msg_watcher(xorriso, NULL, NULL, NULL, NULL, 0);
+     fprintf(stderr, "xorriso -test: Xorriso_start_msg_watcher() = %d\n", ret);
+   } else if(strcmp(arg1, "stop") == 0) {
+     ret= Xorriso_stop_msg_watcher(xorriso, 0);
+     fprintf(stderr, "xorriso -test: Xorriso_stop_msg_watcher() = %d\n", ret);
+
+   } else if(strcmp(arg1, "help") == 0) {
+     fprintf(stderr, "-test [mode] [arguments]\n");
+     fprintf(stderr, "   push\n");
+     fprintf(stderr, "     perform Xorriso_push_outlists()\n");
+     fprintf(stderr, "   pull\n");
+     fprintf(stderr, "     perform Xorriso_pull_outlists() and show messages\n");
+     fprintf(stderr, "   fetch\n");
+     fprintf(stderr, "     perform Xorriso_fetch_outlists() and show\n");
+     fprintf(stderr, "   peek\n");
+     fprintf(stderr, "     perform Xorriso_peek_outlists()\n");
+     fprintf(stderr, "   sleep_peek\n");
+     fprintf(stderr, "     sleep 1 s and perform Xorriso_peek_outlists()\n");
+     fprintf(stderr, "   peek_loop\n");
+     fprintf(stderr, "     wait for up to 3s in Xorriso_peek_outlists()\n");
+     fprintf(stderr, "     for return value 0 or -1\n");
+     fprintf(stderr, "   start\n");
+     fprintf(stderr, "     perform Xorriso_start_msg_watcher()\n");
+     fprintf(stderr, "   stop\n");
+     fprintf(stderr, "     perform Xorriso_stop_msg_watcher()\n");
+   } else {
+     fprintf(stderr, "xorriso -test: unknwon mode: %s\n", arg1);
+   }
+   ret= 0;
+}
+
  } else if(strcmp(cmd,"toc")==0) {
    Xorriso_option_toc(xorriso, 0);
 
+ } else if(strcmp(cmd,"toc_of")==0) {
+   (*idx)++;
+   Xorriso_option_toc_of(xorriso, arg1, 0);
+
  } else if(strcmp(cmd,"uid")==0) {
    (*idx)++;
    ret= Xorriso_option_uid(xorriso,arg1,0);
@@ -1423,6 +1887,13 @@ next_command:;
    (*idx)+= 2;
    ret= Xorriso_option_volume_date(xorriso, arg1, arg2, 0);
 
+ } else if(strcmp(cmd,"write_type")==0) {
+   (*idx)++;
+   ret= Xorriso_option_write_type(xorriso, arg1, 0);
+
+ } else if(strcmp(cmd, "x") == 0) {
+   /* only in effect in Xorriso_prescan_args() */;
+
  } else if(strcmp(cmd,"xattr")==0) {
    (*idx)++;
    ret= Xorriso_option_xattr(xorriso, arg1, 0);
@@ -1438,7 +1909,7 @@ next_command:;
    if(xorriso->add_plainly>1)
      goto add_plain_argument;
 unknown_option:;
-   sprintf(xorriso->info_text, "Not a known option:  '%s'\n",
+   sprintf(xorriso->info_text, "Not a known command:  '%s'\n",
            original_cmd);
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
    {ret= 0; goto eval_any_problems;}
@@ -1459,7 +1930,7 @@ eval_any_problems:
  if(ret<0)
    goto ex;
 
- if(*idx<argc)
+ if(*idx < argc && !(flag & 4))
    goto next_command;
 
 ex:;
@@ -1469,6 +1940,66 @@ ex:;
 }
 
 
+int Xorriso_parse_line(struct XorrisO *xorriso, char *line,
+                       char *prefix, char *separators, int max_words,
+                       int *argc, char ***argv, int flag)
+{
+ int ret, bsl_mode;
+ char *to_parse, *progname= "";
+
+ if(xorriso == NULL && (flag & (32 | 64))) {
+   ret= -2; goto ex;
+ }
+
+ *argc= 0;
+ *argv= NULL;
+
+ to_parse= line;
+ if((flag & 1) || xorriso == NULL)
+   bsl_mode= (flag >> 1) & 3;
+ else
+   bsl_mode= xorriso->bsl_interpretation & 3;
+ if(prefix[0]) {
+   if(strncmp(line, prefix, strlen(prefix)) == 0) {
+     to_parse= line + strlen(prefix);
+   } else {
+     ret= 2; goto ex;
+   }
+ }
+
+ if(xorriso != NULL)
+   progname= xorriso->progname;
+ ret= Sfile_sep_make_argv(progname, to_parse, separators,
+                          max_words, argc, argv,
+                          (!(flag & 32)) | 4 | (bsl_mode << 5));
+ if(ret < 0) {
+   if(xorriso != NULL)
+     Xorriso_msgs_submit(xorriso, 0,
+        "Severe lack of resources during command line parsing", 0, "FATAL", 0);
+   ret= -1; goto ex;
+ }
+ if(ret == 0) {
+   if((flag & 64) && xorriso != NULL) {
+     sprintf(xorriso->info_text, "Incomplete quotation in %s line: %s",
+             (flag & 32) ? "command" : "parsed", to_parse);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+   }
+   goto ex;
+ }
+ ret= 1;
+ex:;
+ if(ret <= 0)
+   Sfile_sep_make_argv("", "", "", 0, argc, argv, 2); /* release memory */
+ return(ret);
+}
+
+
+void Xorriso__dispose_words(int *argc, char ***argv)
+{
+ Sfile_make_argv("", "", argc, argv, 2); /* release memory */
+}
+
+
 int Xorriso_execute_option(struct XorrisO *xorriso, char *line, int flag)
 /*
  bit0-bit15 are forwarded to Xorriso_interpreter
@@ -1488,20 +2019,10 @@ int Xorriso_execute_option(struct XorrisO *xorriso, char *line, int flag)
  xorriso->idle_time= 0.0;
  tdiff= tv.tv_sec+(1.e-6*(double) tv.tv_usec);
 
- /* parse line into args */
- ret= Sfile_make_argv(xorriso->progname, line, &argc, &argv,
-                      4 | ((xorriso->bsl_interpretation & 3) << 5));
- if(ret < 0) {
-   Xorriso_msgs_submit(xorriso, 0,
-      "Severe lack of resources during command line parsing", 0, "FATAL", 0);
+ ret= Xorriso_parse_line(xorriso, line, "", "", 0, &argc, &argv, 32 | 64);
+ if(ret <= 0)
    goto ex;
- }
- if(ret == 0) {
-   sprintf(xorriso->info_text, "Incomplete quotation in command line: %s",
-           line);
-   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
-   goto ex;
- }
+
  if(argc<2)
    {ret= 1; goto ex;}
  if(argv[1][0]=='#')
@@ -1585,10 +2106,31 @@ ex:;
 }
 
 
+/* @return  1=replaced , 2=not replaced , <=0 = error
+*/
+int Xorriso_replace_arg_by_bsl(struct XorrisO *xorriso, char **arg,
+                               char **argpt, int flag)
+{
+ int ret, eaten, l;
+
+ if(!(xorriso->bsl_interpretation & 16))
+   return(2);
+ l= strlen(*argpt);
+ Xorriso_free_meM(*arg);
+ Xorriso_alloc_meM(*arg, char, l + 1);
+ strcpy(*arg, *argpt);
+ *argpt= *arg;
+ ret= Sfile_bsl_interpreter(*arg, l, &eaten, 0);
+ex:;
+ return(ret);
+}
+
+
 int Xorriso_prescan_args(struct XorrisO *xorriso, int argc, char **argv,
                          int flag)
 /*
  bit0= do not interpret argv[1]
+ bit1= complain about inknown arguments
 */
 /*
  return:
@@ -1598,14 +2140,16 @@ int Xorriso_prescan_args(struct XorrisO *xorriso, int argc, char **argv,
 */
 {
  int i, ret, was_dashed, num2, arg_count;
- int advice, mem_add_plainly, error_seen= 0;
+ int advice, mem_add_plainly, error_seen= 0, mem_bsl;
  int was_report_about= 0, was_abort_on= 0, was_return_with= 0;
  int was_signal_handling= 0, was_scsi_log= 0, cmd_data_size= 5 * SfileadrL;
  char *cmd, *original_cmd, *cmd_data= NULL, *arg1, *arg2;
+ char *arg1_data= NULL, *arg2_data= NULL;
  char mem_list_delimiter[81];
 
  strcpy(mem_list_delimiter, xorriso->list_delimiter);
  mem_add_plainly= xorriso->add_plainly;
+ mem_bsl= xorriso->bsl_interpretation;
 
  Xorriso_alloc_meM(cmd_data, char, cmd_data_size);
 
@@ -1619,11 +2163,19 @@ int Xorriso_prescan_args(struct XorrisO *xorriso, int argc, char **argv,
      {ret= -1; goto ex;}
 
    arg1= "";
-   if(i+1<argc)
-     arg1= argv[i+1];
+   if(i + 1 < argc) {
+     arg1= argv[i + 1];
+     ret= Xorriso_replace_arg_by_bsl(xorriso, &arg1_data, &arg1, 0);
+     if(ret <= 0)
+       goto ex;
+   }
    arg2= "";
-   if(i+2<argc)
-     arg2= argv[2+1];
+   if(i + 2 < argc) {
+     arg2= argv[i + 2];
+     ret= Xorriso_replace_arg_by_bsl(xorriso, &arg2_data, &arg2, 0);
+     if(ret <= 0)
+       goto ex;
+   }
    if(i>1)
      xorriso->did_something_useful= 1;
    if(i==1 && argc==2) {
@@ -1750,13 +2302,22 @@ protect_stdout:;
        Xorriso_option_signal_handling(xorriso, arg1, 1); /* no install */
      was_signal_handling= 1;
 
+   } else if(strcmp(original_cmd, "-x") == 0) {
+     xorriso->arrange_args= 1;
+
+   } else if(strcmp(cmd, "backslash_codes") == 0) {
+     i++;
+     ret= Xorriso_option_backslash_codes(xorriso, arg1, 0);
+     if(ret <= 0)
+       error_seen= 1;
+
    } else {
      ret= Xorriso_count_args(xorriso, argc - i, argv + i, &arg_count, 1);
      if(ret == 1) {
        i+= arg_count;
      } else if((flag & 2) && ((was_dashed && xorriso->add_plainly <= 1) ||
                               xorriso->add_plainly <= 0)) {
-       sprintf(xorriso->info_text, "Not a known option:  '%s'\n",
+       sprintf(xorriso->info_text, "Not a known command:  '%s'\n",
                original_cmd);
        Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
        error_seen= 1;
@@ -1767,6 +2328,9 @@ protect_stdout:;
 ex:;
  strcpy(xorriso->list_delimiter, mem_list_delimiter);
  xorriso->add_plainly= mem_add_plainly;
+ xorriso->bsl_interpretation= mem_bsl;
+ Xorriso_free_meM(arg1_data);
+ Xorriso_free_meM(arg2_data);
  Xorriso_free_meM(cmd_data);
  if(error_seen && ret > 0) {
    advice= Xorriso_eval_problem_status(xorriso, 0, 0);
@@ -1967,11 +2531,74 @@ int Xorriso_make_return_value(struct XorrisO *xorriso, int flag)
 int Xorriso_program_arg_bsl(struct XorrisO *xorriso, int argc, char ***argv,
                             int flag)
 {
- int ret;
+ int i, ret, eaten, bsl_mem, params_to_come= 0, cmd_data_size= 5 * SfileadrL;
+ int next_is_backslash_codes= 0, next_is_list_delimiter= 0;
+ char **new_argv= NULL, *cmd, *cmd_data= NULL;
+ char mem_list_delimiter[81];
 
- if(!(xorriso->bsl_interpretation & 16))
-   return(1);
- ret= Sfile_argv_bsl(argc, argv, 0);
+ strcpy(mem_list_delimiter, xorriso->list_delimiter);
+ bsl_mem= xorriso->bsl_interpretation;
+ if(argc <= 0)
+   return(0);
+ Xorriso_alloc_meM(cmd_data, char, cmd_data_size);
+ new_argv= (char **) Smem_malloC(argc * sizeof(char *));
+ if(new_argv == NULL)
+   return(-1); 
+ for(i= 0; i < argc; i++) {
+   new_argv[i]= strdup((*argv)[i]);
+   if(new_argv[i] == NULL)
+     {ret= -1; goto ex;}
+   if(i == 0)
+ continue;
+   if(xorriso->bsl_interpretation & 16) {
+     ret= Sfile_bsl_interpreter(new_argv[i], strlen(new_argv[i]), &eaten, 0);
+     if(ret <= 0)
+       goto ex;
+   }
+   if(params_to_come == 0) {
+     ret= Xorriso_normalize_command(xorriso, new_argv[i], i,
+                                    cmd_data, cmd_data_size, &cmd, 0);
+     if(ret < 0)
+       goto ex;
+     if(strcmp(cmd, "backslash_codes") == 0) {
+       params_to_come= 1;
+       next_is_backslash_codes= 1;
+     } else if(strcmp(cmd, "list_delimiter") == 0) {
+       params_to_come= 1;
+       next_is_list_delimiter= 1;
+     } else {
+       ret= Xorriso_count_args(xorriso, argc - i, *argv + i,
+                               &params_to_come, 1);
+       if(ret <= 0)
+         goto ex;
+       if(ret != 1)
+         params_to_come= 0;
+     }
+   } else {
+     params_to_come--;
+     if(next_is_backslash_codes) {
+       next_is_backslash_codes= 0;
+       ret= Xorriso_option_backslash_codes(xorriso, new_argv[i], 0);
+       if(ret <= 0)
+         goto ex;
+     } else if(next_is_list_delimiter) {
+       next_is_list_delimiter= 0;
+       ret= Xorriso_option_list_delimiter(xorriso, new_argv[i], 0);
+       if(ret <= 0)
+         goto ex;
+     }
+   }
+ }
+ ret= 1;
+ex:;
+ Xorriso_free_meM(cmd_data);
+ strcpy(xorriso->list_delimiter, mem_list_delimiter);
+ xorriso->bsl_interpretation= bsl_mem;
+ if(ret <= 0) {
+   if(new_argv != NULL)
+     free((char *) new_argv);
+ } else
+   *argv= new_argv;
  return(ret);
 }
 
@@ -2232,6 +2859,11 @@ failure:;
    goto failure;
  if(ret > 0)
    hide_attrs|= 2;
+ ret= Exclusions_match(xorriso->hfsplus_hidings, path, 0);
+ if(ret < 0)
+   goto failure;
+ if(ret > 0)
+   hide_attrs|= 4;
  return(hide_attrs);
 }
 
@@ -2388,7 +3020,7 @@ int Xorriso_normalize_acl_text(struct XorrisO *xorriso, char *in_text,
    if(pass == 0) {
      *access_acl_text= calloc(access_count + 1, 1);
      *default_acl_text= calloc(default_count + 1, 1);
-     if(access_acl_text == NULL || *default_acl_text == NULL) {
+     if(*access_acl_text == NULL || *default_acl_text == NULL) {
        Xorriso_no_malloc_memory(xorriso, access_acl_text, 0);
        {ret= -1; goto ex;}
      }
