@@ -2,7 +2,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2012 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2013 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -311,9 +311,37 @@ int Xorriso_set_system_area(struct XorrisO *xorriso, struct burn_drive *drive,
  unsigned char *ub;
  ElToritoBootImage *bootimg;
  IsoFile *bootimg_node;
+ IsoNode *sparc_core_node;
  uint32_t offst;
  enum burn_disc_status state;
 
+ if(xorriso->grub2_sparc_core[0]) {
+   ret= Xorriso_node_from_path(xorriso, img, xorriso->grub2_sparc_core,
+                               &sparc_core_node, 1);
+   if(ret <= 0) {
+     sprintf(xorriso->info_text,
+             "Cannot find in ISO image: -boot_image grub grub2_sparc_core=");
+     Text_shellsafe(xorriso->grub2_sparc_core, xorriso->info_text, 1);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+     {ret= 0; goto ex;}
+   }
+   if(!ISO_NODE_IS_FILE(sparc_core_node)) {
+     sprintf(xorriso->info_text,
+             "Not a data file: -boot_image grub grub2_sparc_core=");
+     Text_shellsafe(xorriso->grub2_sparc_core, xorriso->info_text, 1);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+     {ret= 0; goto ex;}
+   }
+   ret = iso_image_set_sparc_core(img, (IsoFile *) sparc_core_node, 0);
+   if(ret < 0) {
+     Xorriso_process_msg_queues(xorriso,0);
+     Xorriso_report_iso_error(xorriso, "", ret,
+                    "Error when setting up -boot_image grub grub2_sparc_core=",
+                     0, "FAILURE", 1);
+     {ret= 0; goto ex;}
+   }
+ }
+
  Xorriso_alloc_meM(buf, char, 32768);
  system_area_options= xorriso->system_area_options;
  memset(buf, 0, 32768);
@@ -494,7 +522,7 @@ int Xorriso_attach_boot_image(struct XorrisO *xorriso, int flag)
  if(xorriso->boot_efi_default) {
    emul= 0;
    platform_id= 0xef;
-   xorriso->patch_isolinux_image= 0;
+   xorriso->patch_isolinux_image= (xorriso->patch_isolinux_image & ~3) | 0;
  }
  if(platform_id == 0xef || load_size < 0) {
    ret= Xorriso_iso_lstat(xorriso, bin_path, &stbuf, 2 | 4);
@@ -576,6 +604,12 @@ int Xorriso_attach_boot_image(struct XorrisO *xorriso, int flag)
    {ret= 0; goto ex;}
  }
  el_torito_set_boot_platform_id(bootimg, (uint8_t) platform_id);
+ if(load_size / 512 > 65535) {
+   sprintf(xorriso->info_text,
+ "Boot image load size exceeds 65535 blocks. Will record 65535 in El Torito.");
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "WARNING", 0);
+   load_size= 65535 * 512;
+ }
  el_torito_set_load_size(bootimg, load_size / 512); 
  el_torito_set_id_string(bootimg, xorriso->boot_id_string);
  el_torito_set_selection_crit(bootimg, xorriso->boot_selection_crit);
@@ -588,6 +622,7 @@ int Xorriso_attach_boot_image(struct XorrisO *xorriso, int flag)
    xorriso->boot_image_bin_path[0]= 0;
    xorriso->boot_image_bin_form[0]= 0;
    xorriso->boot_image_emul= 0;
+   xorriso->boot_emul_default= 1;
    xorriso->boot_image_load_size= 4 * 512;
    memset(xorriso->boot_id_string, 0, sizeof(xorriso->boot_id_string));
    memset(xorriso->boot_selection_crit, 0,
@@ -605,33 +640,81 @@ ex:;
 }
 
 
+int Xorriso_write_application_use(struct XorrisO *xorriso,
+                                  IsoImage *image, int flag)
+{
+ int l, ret, count= 512;
+ unsigned int byte= 0;
+ char *path, data[512];
+ FILE *fp= NULL;
+
+ path= xorriso->application_use;
+ l= strlen(path);
+ if(l <= 1) {
+   memset(data, path[0], 512);
+ } else if(l == 4 && path[0] == '0' && path[1] == 'x' &&
+           isxdigit(path[2]) && isxdigit(path[3])) {
+   sscanf(path + 2, "%x", &byte);
+   memset(data, (int) byte, 512);
+ } else {
+   /* Read up to 512 bytes from file path */
+   ret= Xorriso_afile_fopen(xorriso, path, "rb", &fp, 0);
+   if(ret <= 0)
+     {ret= 0; goto ex;}
+   ret= fread(data, 1, 512, fp);
+   if(ret < 512) {
+     if(ferror(fp)) {
+       sprintf(xorriso->info_text,
+               "-application_use: Error while reading file ");
+       Text_shellsafe(path, xorriso->info_text, 1);
+       Xorriso_msgs_submit(xorriso, 0, xorriso->info_text,
+                           errno, "FAILURE", 0);
+       ret= 0; goto ex;
+     }
+   }
+   if(ret < 0)
+     count= 0;
+   else
+     count= ret;
+ }
+ iso_image_set_app_use(image, data, count);
+ ret= 1;
+ex:
+ if(fp != NULL && fp != stdin)
+   fclose(fp);
+ return(ret);
+}
+
+
 /* @param flag bit0= do not write but only prepare and return size in sectors
 */
 int Xorriso_write_session(struct XorrisO *xorriso, int flag)
 {
  int ret, relax= 0, i, pacifier_speed= 0, data_lba, ext, is_bootable= 0;
- int freshly_bootable= 0, hide_attr, pad_by_libisofs= 0, signal_mode;
+ int freshly_bootable= 0, hide_attr, pad_by_libisofs= 0, signal_mode, role;
  char *xorriso_id= NULL, *img_id, *sfe= NULL, *out_cs, *part_image;
  struct isoburn_imgen_opts *sopts= NULL;
  struct burn_drive_info *dinfo, *source_dinfo;
  struct burn_drive *drive, *source_drive;
  struct burn_disc *disc= NULL;
- struct burn_write_opts *burn_options;
+ struct burn_write_opts *burn_options= NULL;
  off_t readcounter= 0,writecounter= 0;
  int num_sessions= 0, num_tracks= 0;
  struct burn_session **sessions;
  struct burn_track **tracks;
  enum burn_disc_status s;
+ struct burn_multi_caps *caps= NULL;
  IsoImage *image= NULL;
  IsoNode *root_node;
  int profile_number;
- char *profile_name= NULL;
+ char *profile_name= NULL, *reasons= NULL;
  IsoBoot *bootcat_node;
  uint32_t padding;
 
  Xorriso_alloc_meM(sfe, char, 5 * SfileadrL);
  Xorriso_alloc_meM(xorriso_id, char, 256);
  Xorriso_alloc_meM(profile_name, char, 80);
+ Xorriso_alloc_meM(reasons, char, BURN_REASONS_LEN);
 
  ret= Xorriso_finish_hl_update(xorriso, 0);
  if(ret <= 0)
@@ -641,17 +724,29 @@ int Xorriso_write_session(struct XorrisO *xorriso, int flag)
  if(out_cs == NULL)
    Xorriso_get_local_charset(xorriso, &out_cs, 0);
 
+ ret= Xorriso_get_drive_handles(xorriso, &dinfo, &drive,
+                                "on attempt to write", 2);
+ if(ret<=0)
+   goto ex;
  if(!(flag & 1)) {
    ret= Xorriso_auto_format(xorriso, 0);
    if(ret <=0 )
      {ret= 0; goto ex;}
  }
- ret= Xorriso_get_drive_handles(xorriso, &dinfo, &drive,
-                                "on attempt to write", 2);
- if(ret<=0)
-   goto ex;
 
  s= isoburn_disc_get_status(drive);
+ if (xorriso->do_hfsplus && (
+     (xorriso->grow_blindly_msc2 >= 0 &&
+       xorriso->out_drive_handle != xorriso->in_drive_handle)
+     ||
+     (xorriso->out_drive_handle == xorriso->in_drive_handle && 
+      s != BURN_DISC_BLANK)
+    )) {
+   sprintf(xorriso->info_text,
+           "May not grow ISO image while -hfsplus is enabled");
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+   {ret= 0; goto ex;}
+ }
  if(xorriso->out_drive_handle == xorriso->in_drive_handle) {
    if(abs(xorriso->displacement_sign) == 1 && xorriso->displacement != 0 &&
       s != BURN_DISC_BLANK) {
@@ -709,6 +804,8 @@ int Xorriso_write_session(struct XorrisO *xorriso, int flag)
    iso_image_set_copyright_file_id(image, xorriso->copyright_file);
    iso_image_set_biblio_file_id(image, xorriso->biblio_file);
    iso_image_set_abstract_file_id(image, xorriso->abstract_file);
+   Xorriso_write_application_use(xorriso, image, 0);
+   Xorriso_process_msg_queues(xorriso,0);
  }
 
  if((xorriso->do_aaip & 256) && out_cs != NULL) {
@@ -727,7 +824,7 @@ int Xorriso_write_session(struct XorrisO *xorriso, int flag)
    goto ex;
 
  /* Activate, adjust or discard boot image */
- if(image!=NULL && !(flag&1)) {
+ if(image!=NULL) {
    is_bootable= iso_image_get_boot_image(image, NULL, NULL, &bootcat_node);
    if(xorriso->boot_image_bin_path[0]) {
      ret= Xorriso_attach_boot_image(xorriso, xorriso->boot_count == 0);
@@ -735,6 +832,8 @@ int Xorriso_write_session(struct XorrisO *xorriso, int flag)
        goto ex;
      freshly_bootable= 1;
    }
+ }
+ if(image!=NULL && !(flag&1)) {
    if(xorriso->boot_count > 0) {
      /* Eventually rename boot catalog node to changed boot_image_cat_path */
      if(is_bootable > 0) {
@@ -753,6 +852,8 @@ int Xorriso_write_session(struct XorrisO *xorriso, int flag)
        hide_attr|= LIBISO_HIDE_ON_RR;
      if(xorriso->boot_image_cat_hidden & 2)
        hide_attr|= LIBISO_HIDE_ON_JOLIET;
+     if(xorriso->boot_image_cat_hidden & 4)
+       hide_attr|= LIBISO_HIDE_ON_HFSPLUS;
      iso_image_set_boot_catalog_hidden(image, hide_attr);
    } else if(xorriso->patch_isolinux_image & 1) {
      if(is_bootable == 1) {
@@ -808,13 +909,16 @@ int Xorriso_write_session(struct XorrisO *xorriso, int flag)
  }
 
  isoburn_igopt_set_level(sopts, xorriso->iso_level);
- ext= isoburn_igopt_rockridge |
+ ext= ((!!xorriso->do_rockridge) * isoburn_igopt_rockridge) |
       ((!!xorriso->do_joliet) * isoburn_igopt_joliet) |
+      ((!!xorriso->do_hfsplus) * isoburn_igopt_hfsplus) |
+      ((!!xorriso->do_fat) * isoburn_igopt_fat) |
       ((!!xorriso->do_iso1999) * isoburn_igopt_iso1999) |
       (( !(xorriso->ino_behavior & 2)) * isoburn_igopt_hardlinks) |
       (( (!(xorriso->ino_behavior & 2)) ||
          (xorriso->do_aaip & (2 | 8 | 16 | 256)) ||
-         (xorriso->do_md5 & (2 | 4))
+         (xorriso->do_md5 & (2 | 4)) ||
+         xorriso->do_hfsplus
        ) * isoburn_igopt_aaip) |
       ((!!(xorriso->do_md5 & 2)) * isoburn_igopt_session_md5) |
       ((!!(xorriso->do_md5 & 4)) * isoburn_igopt_file_md5) |
@@ -845,6 +949,16 @@ int Xorriso_write_session(struct XorrisO *xorriso, int flag)
  isoburn_igopt_set_scdbackup_tag(sopts, xorriso->scdbackup_tag_name,
                                  xorriso->scdbackup_tag_time,
                                  xorriso->scdbackup_tag_written);
+ if(xorriso->prep_partition[0]) {
+   ret= isoburn_igopt_set_prep_partition(sopts, xorriso->prep_partition, 0);
+   if(ret <= 0)
+     {ret= 0; goto ex;}
+ }
+ if(xorriso->efi_boot_partition[0]) {
+   ret= isoburn_igopt_set_efi_bootp(sopts, xorriso->efi_boot_partition, 0);
+   if(ret <= 0)
+     {ret= 0; goto ex;}
+ }
  for(i= 0; i < Xorriso_max_appended_partitionS; i++) {
    if(xorriso->appended_partitions[i] == NULL)
  continue;
@@ -858,6 +972,9 @@ int Xorriso_write_session(struct XorrisO *xorriso, int flag)
                                   xorriso->appended_part_types[i], part_image);
  }
  isoburn_igopt_set_disc_label(sopts, xorriso->ascii_disc_label); 
+ isoburn_igopt_set_hfsp_serial_number(sopts, xorriso->hfsp_serial_number);
+ isoburn_igopt_set_hfsp_block_size(sopts, xorriso->hfsp_block_size,
+                                          xorriso->apm_block_size);
 
  if(image!=NULL && 12+strlen(Xorriso_timestamP)<80) {
    strcpy(xorriso_id, xorriso->preparer_id);
@@ -944,6 +1061,10 @@ fprintf(stderr, "XORRISO_DEBUG: isoburn_igopt_set_tail_blocks(%d)\n",
      {ret= 0; goto ex;}
  }
 
+ ret = isoburn_igopt_set_write_type(sopts, xorriso->do_tao);
+ if(ret <= 0)
+   goto ex;
+
  Xorriso_set_abort_severity(xorriso, 1);
  if (xorriso->grow_blindly_msc2 >= 0 &&
      xorriso->out_drive_handle != xorriso->in_drive_handle) {
@@ -993,11 +1114,12 @@ fprintf(stderr, "XORRISO_DEBUG: isoburn_igopt_set_tail_blocks(%d)\n",
  if(ret <= 0)
    goto cancel_iso;
 
+ role= burn_drive_get_drive_role(drive);
+
  /* Important: do not return until burn_is_aborting() was checked */
 
  signal_mode= 1;
- ret= burn_drive_get_drive_role(drive);
- if(ret == 1)
+ if(role == 1)
    signal_mode|= 2;
  Xorriso_set_signal_handling(xorriso, signal_mode);
 
@@ -1009,6 +1131,7 @@ fprintf(stderr, "XORRISO_DEBUG: isoburn_igopt_set_tail_blocks(%d)\n",
  xorriso->run_state= 1; /* Indicate that burning has started */
  isoburn_disc_write(burn_options, disc);
  burn_write_opts_free(burn_options);
+ burn_options= NULL;
 
  ret= Xorriso_pacifier_loop(xorriso, drive, pacifier_speed << 4);
  if(burn_is_aborting(0))
@@ -1073,11 +1196,16 @@ ex:;
  if(image != NULL)
    iso_image_unref(image);
  isoburn_igopt_destroy(&sopts, 0);
+ if(burn_options != NULL)
+   burn_write_opts_free(burn_options);
+ if(caps != NULL)
+   burn_disc_free_multi_caps(&caps);
  Xorriso_process_msg_queues(xorriso,0);
  Xorriso_append_scdbackup_record(xorriso, 0);
  Xorriso_free_meM(sfe);
  Xorriso_free_meM(xorriso_id);
  Xorriso_free_meM(profile_name);
+ Xorriso_free_meM(reasons);
  return(ret);
 
 cancel_iso:;
@@ -1138,7 +1266,8 @@ int Xorriso_pacifier_loop(struct XorrisO *xorriso, struct burn_drive *drive,
  double start_time, current_time, last_time, base_time= 0.0, base_count= 0.0;
  double next_base_time= 0.0, next_base_count= 0.0, first_base_time= 0.0;
  double first_base_count= 0.0, norm= 0.0, now_time, fract_offset= 0.0;
- double measured_speed, speed_factor= 1385000, quot;
+ double measured_speed, speed_factor= 1385000, quot, speed_min_time;
+ double tdiff, now_fract;
  time_t time_prediction;
  IsoImage *image= NULL;
 
@@ -1149,9 +1278,10 @@ int Xorriso_pacifier_loop(struct XorrisO *xorriso, struct burn_drive *drive,
    usleep(100002);
 
  emul= flag&15;
- fract_offset= 0.2 * (double) emul - ((int) (0.2 * (double) emul));
  if(emul==0)
    emul= xorriso->pacifier_style;
+ fract_offset= 1.0 / 3.0 * (double) emul - ((int) (1.0 / 3.0 * (double) emul));
+ speed_min_time= 0.2 * xorriso->pacifier_interval;
  speed_unit= "D";
  if(flag&16) {
    speed_factor= 150.0*1024;
@@ -1186,7 +1316,7 @@ int Xorriso_pacifier_loop(struct XorrisO *xorriso, struct burn_drive *drive,
    }
    current_time= Sfile_microtime(0);
    if(drive_status == BURN_DRIVE_WRITING && progress.sectors > 0) {
-     if(current_time-last_time>0.2)
+     if(current_time-last_time > speed_min_time)
        measured_speed= (progress.sector - last_sector) * 2048.0 /
                        (current_time - last_time);
      buffer_fill= 50;
@@ -1215,7 +1345,7 @@ int Xorriso_pacifier_loop(struct XorrisO *xorriso, struct burn_drive *drive,
        sprintf(xorriso->info_text+strlen(xorriso->info_text), " [buf %3d%%]",
                buffer_fill);
 
-       if(current_time-last_time>0.2)
+       if(current_time-last_time > speed_min_time)
          sprintf(xorriso->info_text+strlen(xorriso->info_text), "  %4.1fx.",
                  measured_speed/speed_factor);
 
@@ -1287,7 +1417,7 @@ int Xorriso_pacifier_loop(struct XorrisO *xorriso, struct burn_drive *drive,
                  "  fifo %3d%%  buf %3d%%",
                  (int) (100.0-100.0*((double) free_bytes)/(double) size),
                  buffer_fill);
-       if(current_time-last_time>0.2)
+       if(current_time - last_time > speed_min_time)
          sprintf(xorriso->info_text+strlen(xorriso->info_text), "  %5.1fx%s ",
                  measured_speed/speed_factor, speed_unit);
      }
@@ -1305,14 +1435,22 @@ int Xorriso_pacifier_loop(struct XorrisO *xorriso, struct burn_drive *drive,
              current_time-start_time);
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "UPDATE", 0);
 
-   for(i= 0; i<12; i++) { /* 2 usleeps more than supposed to be needed */
+   for(i= 0; i < 20; i++) { /* 10 usleeps more than supposed to be needed */
      Xorriso_process_msg_queues(xorriso, 0);
      if(aborting<=0)
        aborting= Xorriso_check_burn_abort(xorriso, 0);
-     usleep(100000);
+     usleep((unsigned long) (100000.0 * xorriso->pacifier_interval));
      now_time= Sfile_microtime(0);
-     if(((time_t) now_time) - ((time_t) current_time) >= 1 &&
-         now_time - ((time_t) now_time) >= fract_offset)
+     tdiff= ((off_t)(now_time / xorriso->pacifier_interval)) -
+            (off_t)(current_time / xorriso->pacifier_interval);
+     now_fract= (now_time / xorriso->pacifier_interval -
+                 (off_t)(now_time / xorriso->pacifier_interval));
+     if(tdiff < 1.0)
+   continue; 
+     if(fract_offset <= 0.0) /* "xorriso" pacifier shall not wait for slot */
+   break;
+     if((now_fract >= fract_offset && now_fract < fract_offset + 0.3) ||
+        tdiff >= 2.0)
    break;
    }
  }
@@ -1447,6 +1585,7 @@ int Xorriso_blank_media(struct XorrisO *xorriso, int flag)
 /* @param flag bit0= try to achieve faster formatting
                bit1= use parameter size (else use default size)
                bit2= do not re-aquire drive
+               bit5= try to disable Defect Management
                bit7= by_index mode:
                      bit8 to bit15 contain the index of the format to use.
    @return 0=failure, did not touch medium , -1=failure, altered medium
@@ -1479,6 +1618,9 @@ int Xorriso_format_media(struct XorrisO *xorriso, off_t in_size, int flag)
  } else {
    mode_flag= 4; /* format to full size */
  }
+ if(flag & 32)
+   mode_flag|= 32; /* try to disable Defect Management */
+ 
  burn_disc_get_profile(drive, &current_profile, current_profile_name);
 
  if(flag&128) { /* by_index */
@@ -1537,7 +1679,7 @@ int Xorriso_format_media(struct XorrisO *xorriso, off_t in_size, int flag)
      mode_flag|= (1<<6);
 
  } else if(current_profile == 0x43) { /* BD-RE */
-   if(!(flag & 2))
+   if(!(flag & (2 | 32)))
      mode_flag= 6; /* format to default payload size */
    if(flag&1)
      mode_flag|= (1<<6);
@@ -1890,14 +2032,37 @@ int Xorriso_burn_track(struct XorrisO *xorriso, off_t write_start_address,
    burn_write_opts_set_start_byte(burn_options, ((off_t) nwa) * (off_t) 2048);
  }
 
- if(burn_write_opts_auto_write_type(burn_options, disc, reasons, 0) ==
-    BURN_WRITE_NONE) {
-   sprintf(xorriso->info_text,
-           "Failed to find a suitable write mode with this media.\n");
-   sprintf(xorriso->info_text+strlen(xorriso->info_text),
-           "Reasons given:\n%s", reasons);
-   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
-   ret= 0; goto ex;
+ if(xorriso->do_tao) {
+   if (xorriso->do_tao > 0)
+     burn_write_opts_set_write_type(burn_options,
+                                    BURN_WRITE_TAO, BURN_BLOCK_MODE1);
+   else
+     burn_write_opts_set_write_type(burn_options,
+                                    BURN_WRITE_SAO, BURN_BLOCK_SAO);
+                                    
+   ret = burn_precheck_write(burn_options, disc, reasons, 0);
+   if(ret<=0) {
+     sprintf(xorriso->info_text,
+             "Cannot set write type %s for this medium.\n",
+             xorriso->do_tao > 0 ? "TAO" : "SAO");
+     sprintf(xorriso->info_text+strlen(xorriso->info_text),
+             "Reasons given:\n%s", reasons);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+     ret= 0; goto ex;
+   }
+   sprintf(xorriso->info_text, "Explicitly chosen write type: %s",
+           xorriso->do_tao > 0 ? "TAO" : "SAO");
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "DEBUG", 0);
+ } else {
+   if(burn_write_opts_auto_write_type(burn_options, disc, reasons, 0) ==
+      BURN_WRITE_NONE) {
+     sprintf(xorriso->info_text,
+             "Failed to find a suitable write mode with this media.\n");
+     sprintf(xorriso->info_text+strlen(xorriso->info_text),
+             "Reasons given:\n%s", reasons);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+     ret= 0; goto ex;
+   }
  }
 
  ret= Xorriso_sanitize_image_size(xorriso, drive, disc, burn_options, 2);
@@ -2011,8 +2176,10 @@ int Xorriso_relax_compliance(struct XorrisO *xorriso, char *mode,
    } else if((l == 16 && strncmp(cpt, "allow_dir_id_ext", l) == 0) ||
              (l == 19 && strncmp(cpt, "allow_dir_id_ext_on", l) == 0) ) {
      xorriso->relax_compliance|= isoburn_igopt_allow_dir_id_ext;
+     xorriso->allow_dir_id_ext_dflt= 0;
    } else if((l == 20 && strncmp(cpt, "allow_dir_id_ext_off", l) == 0)) {
      xorriso->relax_compliance&= ~isoburn_igopt_allow_dir_id_ext;
+     xorriso->allow_dir_id_ext_dflt= 0;
 
    } else if((l == 12 && strncmp(cpt, "omit_version", l) == 0) ||
              (l == 15 && strncmp(cpt, "omit_version_on", l) == 0) ) {
@@ -2141,10 +2308,12 @@ int Xorriso_relax_compliance(struct XorrisO *xorriso, char *mode,
      if(value == 1 || value == 2) {
        limit= ((off_t) 4) * ((off_t) 1024*1024*1024) - ((off_t) 1);
        xorriso->iso_level= value;
+       xorriso->iso_level_is_default= 0;
        if(xorriso->file_size_limit > limit)
          xorriso->file_size_limit= limit;
      } else if(value == 3) {
        xorriso->iso_level= value;
+       xorriso->iso_level_is_default= 0;
        if(xorriso->file_size_limit < Xorriso_default_file_size_limiT)
          xorriso->file_size_limit= Xorriso_default_file_size_limiT;
      } else {
@@ -2286,7 +2455,9 @@ int Xorriso_set_isolinux_options(struct XorrisO *xorriso,
                             "FATAL", 1);
    ret= -1; goto ex;
  }
- patch_table= xorriso->patch_isolinux_image & 1;
+
+ /* bit0 : 1=boot-info-table , bit2-7 : 1=EFI , 2=HFS+ , bit8 : 1=APM */
+ patch_table = xorriso->patch_isolinux_image & 0x3fd;
  if((flag & 1) && num_boots > 1) {
    ret= el_torito_set_isolinux_options(boots[num_boots - 1], patch_table, 0);
    ret= (ret == 1); goto ex;
@@ -2295,13 +2466,13 @@ int Xorriso_set_isolinux_options(struct XorrisO *xorriso,
  /* Handle patching of first attached boot image or of imported boot images 
  */
  for(i= 0; i < num_boots; i++) {
-   patch_table = xorriso->patch_isolinux_image & 1;
+   patch_table = xorriso->patch_isolinux_image & 0x3fd;
    if(patch_table && !(flag & 1)) {
      if(!el_torito_seems_boot_info_table(boots[i], 0))
-       patch_table= 0;
+       patch_table&= ~1;
      else if((xorriso->patch_isolinux_image & 2) &&
              el_torito_get_boot_platform_id(boots[i]) == 0xef)
-       patch_table= 0;
+       patch_table&= ~1;
    }
    if(i > 0 || xorriso->boot_image_isohybrid == 0) {
      ret= el_torito_set_isolinux_options(boots[i], patch_table, 0);
@@ -2357,7 +2528,7 @@ int Xorriso_update_iso_lba0(struct XorrisO *xorriso, int iso_lba, int isosize,
  int ret, full_size, i;
  char *headpt;
  struct burn_drive_info *dinfo;
- struct burn_drive *drive;
+ struct burn_drive *drive = NULL;
  off_t seek_ret, to_write;
  int tag_type;
  uint32_t pos, range_start, range_size, next_tag;
@@ -2387,8 +2558,9 @@ int Xorriso_update_iso_lba0(struct XorrisO *xorriso, int iso_lba, int isosize,
 
  if(!(flag & 2)) {
    /* head_buffer was not filled yet. Read it from output media. */
-   if(burn_drive_get_drive_role(drive) == 5) /* write-only */
-     return(2);
+   if(drive != NULL)
+     if(burn_drive_get_drive_role(drive) == 5) /* write-only */
+       return(2);
    if(job != NULL && job->data_to_fd >= 0) {
      if((flag & 8) && job->sector_map != NULL) {
        ret= Sectorbitmap_bytes_are_set(job->sector_map,
@@ -2416,16 +2588,18 @@ int Xorriso_update_iso_lba0(struct XorrisO *xorriso, int iso_lba, int isosize,
        Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, errno, "FAILURE",0);
        return(0);
      }
-     ret= isoburn_read_iso_head(drive, 0, &isosize, head_buffer, 1 << 13);
+     ret= isoburn_read_iso_head(NULL, 0, &isosize, head_buffer, 1 << 13);
      if(ret<=0) {
        Xorriso_process_msg_queues(xorriso,0);
        sprintf(xorriso->info_text,
-               "Alleged session start does not like ISO 9660.");
+               "Alleged session start does not look like ISO 9660.");
        Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, errno, "FAILURE",0);
        return(0);
      }
    } else {
-     ret= isoburn_read_iso_head(drive, iso_lba, &isosize, head_buffer, 2);
+     ret= 0;
+     if(drive != NULL)
+       ret= isoburn_read_iso_head(drive, iso_lba, &isosize, head_buffer, 2);
      if(ret<=0) {
        Xorriso_process_msg_queues(xorriso,0);
        sprintf(xorriso->info_text,
