@@ -67,8 +67,8 @@ struct XorrisO;
     @since 0.5.8
 */
 #define Xorriso_header_version_majoR  1
-#define Xorriso_header_version_minoR  2
-#define Xorriso_header_version_micrO  8
+#define Xorriso_header_version_minoR  3
+#define Xorriso_header_version_micrO  2
 
 
 /** If needed: Something like ".pl01" to indicate a bug fix. Normally empty.
@@ -241,7 +241,7 @@ int Xorriso_destroy(struct XorrisO **xorriso, int flag);
 /* This special interpreter may be called between Xorriso_new() and
    Xorriso_startup_libraries(). It interprets certain commands which shall
    get into effect before the libraries get initialized:
-     -abort_on , -report_about , -return_with , -list_delimiter ,
+     -abort_on , -report_about , -return_with ,
      -scsi_log , -signal_handling
    This is the only occasion where command -x has an effect:
      -x
@@ -251,9 +251,10 @@ int Xorriso_destroy(struct XorrisO **xorriso, int flag);
      -no_rc
    Some get examined for the need to redirect stdout messages:
      -dev , -outdev , -indev , -as 
-   Commands -list_delimiter and -add_plainly get into effect during this
-   call. But their setting at begin of the call gets restored before the
-   call returns.
+   Commands
+     -backslash_codes , -list_delimiter , -add_plainly
+   get into effect during this call. But their setting at begin of the call
+   gets restored before the call returns.
    @param xorriso The context object in which to perform the commands.
    @param argc    Number of arguments.
    @param argv    The arguments. argv[0] contains the program name.
@@ -288,6 +289,17 @@ int Xorriso_read_rc(struct XorrisO *xorriso, int flag);
    replace *argv by a new argument vector. The old one will not be freed
    by this call. If it is dynamic memory then you need to keep a copy of
    the pointer and free it yourself after this call.
+
+   @since 1.3.2:
+   This call internally interprets the commands -backslash_codes and
+   -list_delimiter if it encounters them among the arguments. The
+   decoding of backslashes can thus be enabled and disabled by the
+   arguments themselves. The state of the xorriso object in respect
+   to these commands gets preserved at the start of the call and restored
+   when the call ends.
+   (*argv)[0] never gets decoded.
+   The old *argv will always be replaced by a new one.
+
    @param xorriso The context object
    @param argc    Number of arguments.
    @param argv    The arguments. (*argv)[0] contains the program name.
@@ -1241,6 +1253,11 @@ int Xorriso_option_append_partition(struct XorrisO *xorriso, char *partno_text,
 int Xorriso_option_application_id(struct XorrisO *xorriso, char *name, 
                                   int flag);
 
+/* Command -application_use */
+/* @since 1.3.2 */
+int Xorriso_option_application_use(struct XorrisO *xorriso, char *path,
+                                   int flag);
+
 /* Command -as */
 /* @param flag bit0=do not report the added item
                bit1=do not reset pacifier, no final pacifier message
@@ -1585,10 +1602,10 @@ int Xorriso_option_logfile(struct XorrisO *xorriso, char *channel,
 /* Command -ls  alias -lsi   and -lsl  alias -lsli
        and -lsd alias -lsdi  and -lsdl alias -lsdli
        and -du  alias -dui   and -dus  alias -dusi
-   @param flag bit0= long format (-lsl , -du)
+   @param flag bit0= long format (-lsl , -du, not -dus, not -ls)
                bit1= do not expand patterns but use literally
-               bit2= du rather than ls
-               bit3= list directories as themselves (ls -d)
+               bit2= -du rather than -ls
+               bit3= list directories as themselves (-lsd)  
 */
 int Xorriso_option_lsi(struct XorrisO *xorriso, int argc, char **argv,
                       int *idx, int flag);
@@ -1651,6 +1668,12 @@ int Xorriso_option_msg_op(struct XorrisO *xorriso, char *what, char *arg,
 int Xorriso_option_mvi(struct XorrisO *xorriso, int argc, char **argv,
                       int *idx, int flag);
 
+/* Option -named_pipe_loop */
+/* @since 1.3.2 */
+int Xorriso_option_named_pipe_loop(struct XorrisO *xorriso, char *mode,
+                                   char *stdin_pipe, char *stdout_pipe,
+                                   char *stderr_pipe, int flag);
+
 /* Command -no_rc */
 int Xorriso_option_no_rc(struct XorrisO *xorriso, int flag);
 
@@ -1832,6 +1855,11 @@ int Xorriso_option_setfattr_listi(struct XorrisO *xorriso, char *path,
 int Xorriso_option_set_filter(struct XorrisO *xorriso, char *name,
                               int argc, char **argv, int *idx, int flag);
 
+/* Option -sh_style_result */
+/* @since 1.3.2 */
+int Xorriso_option_sh_style_result(struct XorrisO *xorriso, char *mode,
+                                   int flag);
+
 /* Command -signal_handling */
 /* @param flag bit0= do not yet install the eventual handler
    @since 1.1.0
