@@ -2,7 +2,7 @@
 /* Command line oriented batch and dialog tool which creates, loads,
    manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2012 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2013 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -19,6 +19,12 @@
 #define Xorriso_private_includeD yes
 
 
+/* <<< Disable this to disable pthread_mutex locking on message and result
+       output.
+*/
+#define Xorriso_fetch_with_msg_queueS yes
+
+
 /* for uint32_t */
 #ifdef HAVE_STDINT_H
 #include <stdint.h>
@@ -129,9 +135,12 @@ struct XorrisO { /* the global context of xorriso */
                             Do not try to detect hardlink splits and joinings.
                       bit4= with extract:
                             Do not create or use hln arrays if sort_lba_on
+                      bit5= with command -lsl
+                            Do not create hln array for hard link count
                     */
 
  int iso_level;
+ int iso_level_is_default;
  int do_joliet;
  int do_hfsplus;
  int do_fat;
@@ -173,6 +182,7 @@ struct XorrisO { /* the global context of xorriso */
  char scdbackup_tag_listname[SfileadrL];
 
  int relax_compliance;        /* opaque bitfield to be set by xorrisoburn */
+ int allow_dir_id_ext_dflt; /* -compliance allow_dir_id_ext still on default */
  char rr_reloc_dir[256];
  int rr_reloc_flags;
  int untranslated_name_len;
@@ -222,6 +232,8 @@ struct XorrisO { /* the global context of xorriso */
  char biblio_file[38];
  char abstract_file[38];
 
+ char application_use[SfileadrL];
+
  char session_logfile[SfileadrL];
  int session_lba;
  int session_blocks;
@@ -238,6 +250,8 @@ struct XorrisO { /* the global context of xorriso */
                                   do not emulate TOC on overwriteable media
                             bit2= bit7 for isoburn_drive_aquire()
                                   pretend any media to be -ROM
+                            bit3= bit9 for isoburn_drive_aquire()
+                                  Ignore enclosing session at LBA 0
                          */
 
  int image_start_mode;  /* From what address to load the ISO image
@@ -354,11 +368,13 @@ struct XorrisO { /* the global context of xorriso */
                               bit2-7= Mentioning in isohybrid GPT
                                       1=EFI, 2=HFS+
                               bit8= Mention in isohybrid Apple Partition Map
+                              bit9= GRUB2 boot provisions (patch at byte 1012)
                             */
  int boot_image_emul; /* 0=no emulation
                          1=emulation as hard disk
                          2=emulation as floppy
                       */
+ int boot_emul_default;  /* 1= boot_image_emul is still default */
  off_t boot_image_load_size;
  unsigned char boot_id_string[29];
  unsigned char boot_selection_crit[21];
@@ -394,6 +410,14 @@ struct XorrisO { /* the global context of xorriso */
                                                    any other feature which
                                                    needs to have own MBR
                                                    partition entries.
+                                         bit14= Only with System area type 0
+                                               GRUB2 boot provisions:
+                                               Patch system area at byte 92 to
+                                               99 with 512-block address + 1
+                                               of the first boot image file.
+                                               Little-endian 8-byte.
+                                               Should be combined with
+                                               options bit0.
                                        */
  int patch_system_area;               /* Bits as of system_area_options.
                                          to be applied to the loaded system
@@ -423,6 +447,11 @@ struct XorrisO { /* the global context of xorriso */
  */
  char ascii_disc_label[Xorriso_disc_label_sizE];
 
+ /* A data file of which the position and size shall be written after
+    a SUN Disk Label.
+ */
+ char grub2_sparc_core[SfileadrL];
+
  /* HFS+ image serial number.
     00...00 means that it shall be generated by libisofs.
  */
@@ -465,7 +494,8 @@ struct XorrisO { /* the global context of xorriso */
  /* XORRISO options */
  int allow_graft_points;
 
- int allow_restore; /* -1=permanently disallowed
+ int allow_restore; /* -2=disallowed until special mode "unblock"
+                       -1=permanently disallowed
                         0=disallowed, 1=allowed, 2=device files allowed */
  int do_concat_split;  /* 1= restore complete split file directories as
                              regular files
@@ -492,6 +522,8 @@ struct XorrisO { /* the global context of xorriso */
 
  int dialog;  /* 0=off , 1=single-line , 2=multi-line */
 
+ struct Xorriso_lsT *buffered_dialog; /* If not NULL : read by dialog */
+
  int bsl_interpretation;
                      /* whether to run input through Sfile_bsl_interpreter():
                         bit0-1= dialog and quoted file reading
@@ -505,6 +537,14 @@ struct XorrisO { /* the global context of xorriso */
                         bit6= perform backslash encoding with info texts
                      */
 
+ int sh_style_result; /* Whether not to wrap into quotation marks the file
+                         addresses reported by:
+                           pwd pwdx ls lsd lsl lsdl lsx lsdx lslx lsdlx
+                           du dus dux dusx findx find
+                         and to make du* numbers left adjusted,
+                         and not to append "/" to pwd* 
+                       */
+
  /* Pattern matching facility. It still carries legacy from scdbackup/askme.c
     but is fully functional for xorriso.
  */
@@ -544,6 +584,8 @@ struct XorrisO { /* the global context of xorriso */
  struct ExclusionS *hfsplus_hidings;
 
  int use_stdin; /* use raw stdin even if readline support is compiled */
+ int tolerate_stdin_eof;  /* Do not abort on EOF in Xorriso_dialog_input
+                             but rather return -2. */
  int result_page_length;
  int result_page_width;
  char mark_text[SfileadrL]; /* ( stdout+stderr, M: ) */
@@ -559,6 +601,33 @@ struct XorrisO { /* the global context of xorriso */
                                                */
  int msglist_stackfill;
 
+ int lib_msg_queue_lock_ini;
+ int result_msglists_lock_ini;
+ pthread_mutex_t lib_msg_queue_lock;
+ pthread_mutex_t result_msglists_lock;
+
+ int write_to_channel_lock_ini;
+ pthread_mutex_t write_to_channel_lock;
+
+ int msg_watcher_lock_ini;
+ pthread_mutex_t msg_watcher_lock;
+ int msg_watcher_state; /* 0= inactive
+                           1= registered
+                           2= started
+                           3= request to end
+                        */
+ int (*msgw_result_handler)(void *handle, char *text);
+ void *msgw_result_handle;
+ int (*msgw_info_handler)(void *handle, char *text);
+ void *msgw_info_handle;
+ int msgw_stack_handle;
+ int msgw_msg_pending; /* 0=no, 1=fetching(i.e. maybe) , 2=yes */
+ int msgw_fetch_lock_ini;
+ pthread_mutex_t msgw_fetch_lock;
+
+ struct Xorriso_msg_sievE *msg_sieve;
+ int msg_sieve_disabled;
+
  int status_history_max; /* for -status long_history */
  
  /* 0= no logging of SCSI commands, 1= to stderr */
@@ -572,6 +641,8 @@ struct XorrisO { /* the global context of xorriso */
  int abort_on_is_default; /* will be set to 0 by first -abort_on */
  int problem_status; /* Severity rank number. 0= no abort condition present */
  char problem_status_text[20];
+ int problem_status_lock_ini;
+ pthread_mutex_t problem_status_lock;
 
  char errfile_log[SfileadrL]; /* for -errfile_log */
  int errfile_mode; /* bit0= marked */
@@ -607,6 +678,7 @@ struct XorrisO { /* the global context of xorriso */
  double insert_count;
  double insert_bytes;
  double error_count; /* double will not roll over */
+ int launch_frontend_banned;
 
  /* pacifiers */
  int pacifier_style; /* 0= xorriso, 1=mkisofs 2=cdrecord */
