@@ -4,9 +4,9 @@
    a command line oriented batch and dialog tool which creates, loads,
    manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2009 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2013 Thomas Schmitt, <scdbackup@gmx.net>
 
-   Provided under GPL version 2.
+   Provided under GPL version 2 or later.
 
    This file contains the inner isofs- and burn-library interface of xorriso.
 */
@@ -17,9 +17,15 @@
 
 /* The minimum version of libisoburn to be used with this version of xorriso
 */
-#define xorriso_libisoburn_req_major  0
+#define xorriso_libisoburn_req_major  1
 #define xorriso_libisoburn_req_minor  3
-#define xorriso_libisoburn_req_micro  4
+#define xorriso_libisoburn_req_micro  2
+
+
+struct SpotlisT;          /* List of intervals with different read qualities */
+struct CheckmediajoB;     /* Parameters for Xorriso_check_media() */
+struct Xorriso_msg_sievE; /* Fishes for info particles in reply messages */
+
 
 int Xorriso_startup_libraries(struct XorrisO *xorriso, int flag);
 
@@ -32,7 +38,8 @@ int Xorriso_create_empty_iso(struct XorrisO *xorriso, int flag);
                bit1=aquire as libburn output drive (as isoburn drive if bit0)
    @return <=0 failure , 1=success , 2=neither readable or writeable
 */
-int Xorriso_aquire_drive(struct XorrisO *xorriso, char *adr, int flag);
+int Xorriso_aquire_drive(struct XorrisO *xorriso, char *adr, char *show_adr,
+                         int flag);
 
 int Xorriso_give_up_drive(struct XorrisO *xorriso, int flag);
 
@@ -50,6 +57,8 @@ int Xorriso_graft_in(struct XorrisO *xorriso, void *boss_iter,
 
 int Xorriso__text_to_sev(char *severity_name, int *severity_number,int flag);
 
+int Xorriso__sev_to_text(int severity, char **severity_name, int flag);
+
 /* @param flag bit0=report about output drive 
                bit1=short report form
                bit2=do not try to read ISO heads
@@ -68,8 +77,8 @@ int Xorriso_tell_media_space(struct XorrisO *xorriso,
                              int *media_space, int *free_space, int flag);
 
 /* @param flag bit0=fast , bit1=deformat
-   @return 0=failure, did not touch media , -1=failure, altered media
-           1=success, altered media       ,  2=success, did not touch media
+   @return 0=failure, did not touch medium , -1=failure, altered medium
+           1=success, altered medium       ,  2=success, did not touch medium
 */
 int Xorriso_blank_media(struct XorrisO *xorriso, int flag);
 
@@ -78,8 +87,8 @@ int Xorriso_blank_media(struct XorrisO *xorriso, int flag);
                bit2= do not re-aquire drive
                bit7= by_index mode:
                      bit8 to bit15 contain the index of the format to use.
-   @return 0=failure, did not touch media , -1=failure, altered media
-           1=success, altered media       ,  2=success, did not touch media
+   @return 0=failure, did not touch medium , -1=failure, altered medium
+           1=success, altered medium       ,  2=success, did not touch medium
 */
 int Xorriso_format_media(struct XorrisO *xorriso, off_t size, int flag);
 
@@ -87,9 +96,18 @@ int Xorriso_format_media(struct XorrisO *xorriso, off_t size, int flag);
 */
 int Xorriso_list_formats(struct XorrisO *xorriso, int flag);
 
+/* @return <=0 error, 1 success
+*/ 
+int Xorriso_list_speeds(struct XorrisO *xorriso, int flag);
+
+/* @param flag bit1= obtain outdrive, else indrive
+   @return <=0 error, 1 success
+*/
+int Xorriso_list_profiles(struct XorrisO *xorriso, int flag);
+
 /* @param flag bit2= formatting rather than blanking
-   @return 0=failure, did not touch media , -1=failure, altered media
-           1=success, altered media       ,  2=success, did not touch media
+   @return 0=failure, did not touch medium , -1=failure, altered medium
+           1=success, altered medium       ,  2=success, did not touch medium
 */ 
 int Xorriso_blank_as_needed(struct XorrisO *xorriso, int flag);
 
@@ -102,6 +120,9 @@ int Xorriso_blank_as_needed(struct XorrisO *xorriso, int flag);
                bit1= remove empty directory: rmdir
                bit2= recursion: do not reassure in mode 2 "tree"
                bit3= this is for overwriting and not for plain removal
+               bit4= count deleted files in xorriso->pacifier_count
+               bit5= with bit0 only remove directory content, not the directory
+               bit6= do not delete eventually existing node from di_array
    @return   <=0 = error
                1 = removed simple node
                2 = removed directory or tree
@@ -110,6 +131,7 @@ int Xorriso_blank_as_needed(struct XorrisO *xorriso, int flag);
 int Xorriso_rmi(struct XorrisO *xorriso, void *boss_iter, off_t boss_mem,
                 char *path, int flag);
 
+
 /* @param flag bit0= long format
                bit1= do not print count of nodes
                bit2= du format
@@ -177,6 +199,12 @@ int Xorriso_findi(struct XorrisO *xorriso, struct FindjoB *job,
                   void *dir_node_generic, char *dir_path,
                   struct stat *dir_stbuf, int depth, int flag);
 
+/* @param flag bit0= do not dive into trees
+               bit1= do not perform job->action on resulting node array
+*/
+int Xorriso_findi_sorted(struct XorrisO *xorriso, struct FindjoB *job,
+                         off_t boss_mem, int filec, char **filev, int flag);
+
 /* @param flag bit0= do not mark image as changed */
 int Xorriso_set_volid(struct XorrisO *xorriso, char *volid, int flag);
 
@@ -211,17 +239,8 @@ int Xorriso_burn_track(struct XorrisO *xorriso, off_t write_start_address,
 int Xorriso_get_profile(struct XorrisO *xorriso, int *profile_number,
                         char profile_name[80], int flag);
 
-#ifdef NIX
-
-/* @param flag bit0= do not mark image as changed */
-int Xorriso_set_publisher(struct XorrisO *xorriso, char *name, int flag);
-
-/* @param flag bit0= do not mark image as changed */
-int Xorriso_set_application_id(struct XorrisO *xorriso, char *name, int flag);
-
-#endif /* NIX */
-
 /* @param flag bit0= node_pt is a valid ISO object handle, ignore pathname
+               bit1= dig out the most original stream for reading
 */
 int Xorriso_iso_file_open(struct XorrisO *xorriso, char *pathname,
                           void *node_pt, void **stream, int flag);
@@ -308,50 +327,31 @@ int Xorriso_get_local_charset(struct XorrisO *xorriso, char **name, int flag);
 
 int Xorriso_set_local_charset(struct XorrisO *xorriso, char *name, int flag);
 
+int Xorriso_destroy_node_array(struct XorrisO *xorriso, int flag);
 
+int Xorriso_destroy_hln_array(struct XorrisO *xorriso, int flag);
 
-struct CheckmediajoB {
- int use_dev;        /* 0= use indev , 1= use outdev , 2= use sector map*/
+int Xorriso_destroy_di_array(struct XorrisO *xorriso, int flag);
 
- int min_lba;        /* if >=0 : begin checking at this address */
- int max_lba;        /* if >=0 : read up to this address, else use mode */
+int Xorriso_new_node_array(struct XorrisO *xorriso, off_t mem_limit,
+                           int addon_nodes, int flag);
 
- int min_block_size; /* >>> not yet implemented:
-                            granularity desired by user
-                     */
- int mode;           /* 0= track by track
-                        1= single sweep over libisoburn media capacity
-                        >>> 2= single sweep over libburn media capacity
-                     */
- time_t start_time;
- int time_limit;     /* Number of seconds after which to abort */
+int Xorriso_sort_node_array(struct XorrisO *xorriso, int flag);
 
- int item_limit;     /* Maximum number of media check list items as result */
+int Xorriso_new_hln_array(struct XorrisO *xorriso, off_t mem_limit, int flag);
 
- char abort_file_path[SfileadrL];
+/* @param flag bit0= allocate xorriso->node_targets too
+*/
+int Xorriso_restore_node_array(struct XorrisO *xorriso, int flag);
 
- char data_to_path[SfileadrL];
- int data_to_fd;
- off_t data_to_offset; /* usually 0 with image copy, negative with file copy */
- off_t data_to_limit;  /* used with file copy */
- int patch_lba0;
- int patch_lba0_msc1;
+int Xorriso_check_md5(struct XorrisO *xorriso, void *in_node, char *path,
+                      int flag);
 
- char sector_map_path[SfileadrL];
- struct SectorbitmaP *sector_map;
- int map_with_volid;  /* 0=add quick toc to map file,
-                         1=read ISO heads for toc
-                      */
+int Xorriso_check_session_md5(struct XorrisO *xorriso, char *severity,
+                              int flag);
 
- int retry;          /* -1= only try full read_chunk, 1=retry with 2k blocks
-                         0= retry with CD, full chunk else
-                      */
+int Xorriso_image_has_md5(struct XorrisO *xorriso, int flag);
 
- int report_mode;    /* 0= print MCL items
-                        1= print damaged files
-                     */
-
-};
 
 int Xorriso_check_media(struct XorrisO *xorriso, struct SpotlisT **spotlist,
                         struct CheckmediajoB *job, int flag);
@@ -401,6 +401,14 @@ int Xorriso_getfacl(struct XorrisO *xorriso, void *node,
 int Xorriso_getfattr(struct XorrisO *xorriso, void *in_node, char *path,
                      char **attr_text, int flag);
 
+int Xorriso_list_extattr(struct XorrisO *xorriso, void *in_node, char *path,
+                         char *show_path, char *mode, int flag);
+
+int Xorriso_append_extattr_comp(struct XorrisO *xorriso,
+                                char *comp, size_t comp_len, 
+                                char *mode, int flag);
+
+
 /* Calls iso_image_set_ignore_aclea() according to  xorriso->do_aaip */
 int Xorriso_set_ignore_aclea(struct XorrisO *xorriso, int flag);
 
@@ -424,6 +432,15 @@ int Xorriso_setfattr(struct XorrisO *xorriso, void *in_node, char *path,
                      size_t num_attrs, char **names,
                      size_t *value_lengths, char **values, int flag);
 
+int Xorriso_perform_attr_from_list(struct XorrisO *xorriso, char *path,
+                      struct Xorriso_lsT *lst_start, int flag);
+
+int Xorriso_path_setfattr(struct XorrisO *xorriso, void *in_node, char *path,
+                       char *name, size_t value_length, char *value, int flag);
+
+int Xorriso_perform_acl_from_list(struct XorrisO *xorriso, char *file_path,
+                                  char *uid, char *gid, char *acl, int flag);
+
 int Xorriso_record_dev_inode(struct XorrisO *xorriso, char *disk_path,
                              dev_t dev, ino_t ino,
                              void *in_node, char *iso_path, int flag);
@@ -431,6 +448,97 @@ int Xorriso_record_dev_inode(struct XorrisO *xorriso, char *disk_path,
 int Xorriso_local_getfacl(struct XorrisO *xorriso, char *disk_path,
                           char **text, int flag);
 
+int Xorriso_set_filter(struct XorrisO *xorriso, void *in_node,
+                       char *path, char *filter_name, int flag);
+
+/* @param flag bit0= delete filter with the given name
+*/
+int Xorriso_external_filter(struct XorrisO *xorriso,
+                            char *name, char *options, char *path,
+                            int argc, char **argv, int flag);
+
+int Xorriso_status_extf(struct XorrisO *xorriso, char *filter, FILE *fp,
+                        int flag);
+
+int Xorriso_destroy_all_extf(struct XorrisO *xorriso, int flag);
+
+int Xorriso_show_stream(struct XorrisO *xorriso, void *in_node,
+                        char *path, int flag);
+
+int Xorriso_set_zisofs_params(struct XorrisO *xorriso, int flag);
+
+int Xorriso_status_zisofs(struct XorrisO *xorriso, char *filter, FILE *fp,
+                          int flag);
+
+/* @param flag bit0= overwrite existing di_array (else return 2)
+*/
+int Xorriso_make_di_array(struct XorrisO *xorriso, int flag);
+
+/* @param flag bit0= overwrite existing hln_array (else return 2)
+*/
+int Xorriso_make_hln_array(struct XorrisO *xorriso, int flag);
+
+/*
+   @param flag      bit2= -follow: this is not a command parameter
+   @return -1= severe error
+            0= not applicable for hard links
+            1= go on with processing
+            2= iso_rr_path is fully updated
+*/
+int Xorriso_hardlink_update(struct XorrisO *xorriso, int *compare_result,
+                            char *disk_path, char *iso_rr_path, int flag);
+
+int Xorriso_finish_hl_update(struct XorrisO *xorriso, int flag);
+
+int Xorriso_get_md5(struct XorrisO *xorriso, void *in_node, char *path,
+                    char md5[16], int flag);
+
+int Xorriso_make_md5(struct XorrisO *xorriso, void *in_node, char *path,
+                     int flag);
+
+int Xorriso_md5_start(struct XorrisO *xorriso, void **ctx, int flag);
+
+int Xorriso_md5_compute(struct XorrisO *xorriso, void *ctx,
+                        char *data, int datalen, int flag);
+
+int Xorriso_md5_end(struct XorrisO *xorriso, void **ctx, char md5[16],
+                    int flag);
+
+/* @param flag bit0=input drive
+               bit1=output drive
+*/
+int Xorriso_drive_snooze(struct XorrisO *xorriso, int flag);
+
+int Xorriso_is_plain_image_file(struct XorrisO *xorriso, void *in_node,
+                                char *path, int flag);
+
+int Xorriso_pvd_info(struct XorrisO *xorriso, int flag);
+
+/* @param flag bit0= do not set hln_change_pending */
+int Xorriso_set_change_pending(struct XorrisO *xorriso, int flag);
+
+/* @param flag bit0= enable SCSI command logging to stderr */
+int Xorriso_scsi_log(struct XorrisO *xorriso, int flag);
+
+/* flag bit0= do not increment boot_count and do not reset boot parameters
+        bit1= dispose attached boot images
+*/
+int Xorriso_attach_boot_image(struct XorrisO *xorriso, int flag);
+
+/*
+ bit0= do only report non-default settings
+ bit1= do only report to fp
+*/
+int Xorriso_boot_image_status(struct XorrisO *xorriso, char *filter, FILE *fp,
+                              int flag);
+
+int Xorriso_add_mips_boot_file(struct XorrisO *xorriso, char *path, int flag);
+
+int Xorriso_coordinate_system_area(struct XorrisO *xorriso, int sa_type,
+                                   int options, char *cmd, int flag);
+
+
+
 
 /* A pseudo file type for El-Torito bootsectors as in man 2 stat :
    For now take the highest possible value.
@@ -438,5 +546,71 @@ int Xorriso_local_getfacl(struct XorrisO *xorriso, char *disk_path,
 #define Xorriso_IFBOOT S_IFMT
 
 
+
+int Exprtest_match(struct XorrisO *xorriso, struct ExprtesT *ftest,
+                   void *node_pt, char *name, char *path,
+                   struct stat *boss_stbuf, struct stat *stbuf, int flag);
+
+
+int Xorriso_toc_to_string(struct XorrisO *xorriso, char **toc_text, int flag);
+
+
+int Xorriso_reaquire_outdev(struct XorrisO *xorriso, int flag);
+
+int Xorriso_set_system_area_path(struct XorrisO *xorriso, char *path,
+                                 int flag);
+
+int Xorriso_set_hidden(struct XorrisO *xorriso, void *in_node, char *path,
+                       int hide_state, int flag);
+
+
+/* @param flag  bit0= avoid library calls
+ */
+int Xorriso_preparer_string(struct XorrisO *xorriso, char xorriso_id[129],
+                            int flag);
+
+int Xorriso_jigdo_interpreter(struct XorrisO *xorriso, char *aspect, char *arg,
+                              int flag);
+
+
+int Xorriso_estimate_file_size(struct XorrisO *xorriso, struct FindjoB *job,
+                      char *basename, mode_t st_mode, off_t st_size, int flag);
+
+
+int Xorriso_clone_tree(struct XorrisO *xorriso, void *boss_iter,
+                       char *origin, char *dest, int flag);
+
+int Xorriso_clone_under(struct XorrisO *xorriso, char *origin, char *dest,
+                        int flag);
+
+int Xorriso_mark_update_merge(struct XorrisO *xorriso, char *path,
+                              void *node, int flag);
+
+/* @param flag bit0= asynchronous handling (else catch thread, wait, and exit)
+*/
+int Xorriso_set_signal_handling(struct XorrisO *xorriso, int flag);
+
+/* @param flag bit0=force burn_disc_close_damaged()
+*/
+int Xorriso_close_damaged(struct XorrisO *xorriso, int flag);
+
+int Xorriso_list_extras(struct XorrisO *xorriso, char *mode, int flag);
+
+int Xorriso_set_data_cache(struct XorrisO *xorriso, void *ropts,
+                           int num_tiles, int tile_blocks, int flag);
+
+int Xorriso_hfsplus_file_creator_type(struct XorrisO *xorriso, char *path,
+                                      void *in_node, 
+                                      char *creator, char *hfs_type, int flag); 
+
+int Xorriso_hfsplus_bless(struct XorrisO *xorriso, char *path,
+                          void *in_node, char *blessing, int flag);
+
+int Xorriso_pretend_full_disc(struct XorrisO *xorriso, int flag);
+
+
+int Xorriso_apply_sort_file(struct XorrisO *xorriso, char *path, int flag);
+
+
 #endif /* Xorrisoburn_includeD */
 
